﻿Add-PSSnapin microsoft.sharepoint.powershell
Import-Module .\SP2013_GlobalFunctions.psm1 -AsCustomObject -Force -DisableNameChecking

#region Service_Applicaton_Vars

#In this simple version all service apps will be provisioned in the same app pool
#All databases will be provisioned on the same shared SQL instance

$SharedAppPoolName = "SharePoint Web Services Default"
$sharedAppPoolAcct = "richsp\svc_spservice"   #Can be Edited
$sharedDatabaseServerName = "SQLCL01-AG01-L01"   #Can be Edited

# USAGE & HEALTH SERVICE APPLICATION SETTINGS

    # service app display name
    $usageHealthSvcDisplayName = "Usage and Health Service Application"

    # database settings
    $usageHealthSvcDatabaseName   = "CPS_SP2013_SA_USAGE_HEALTH"
   

# MANAGED METADATA SERVICE APPLICATION SETTINGS

    # service app display name
    $mmSvcDisplayName    = "Managed Metadata Service Application"

    # database settings
    $mmSvcDatabaseName   = "CPS_SP2013_SA_MMS"


# SECURE STORE SERVICE APPLICATION SETTTINGS

    # service app display name
    $secureStoreSvcDisplayName = "Secure Store Service Application"

    # database settings
    $secureStoreSvcDatabaseName   = "CPS_SP2013_SA_SECURE_STORE"

# USER PROFILE SERVICE APPLICATION SETTINGS

    # service app display name
    $profileSvcDisplayName = "User Profile Service Application"

    # MySite/Personal Site host URL.  Set to "" or $NULL to skip setting of the value on UPA
    $personalSiteUrl = $null # "https://personal.contoso.com"

    # database settings
    $upaProfileDBName   = "CPS_SP2013_SA_UPA_PROFILE"
    $upaSocialDBName    = "CPS_SP2013_SA_UPA_SOCIAL"
    $upaSyncDBName      = "CPS_SP2013_SA_UPA_SYNC"


# STATE SERVICE SERVICE APPLICATION SETTINGS

    # service app display name
    $stateServiceSvcDisplayName = "State Service Service Application"

    # database settings
    $stateServiceSvcDatabaseName   = "CPS_SP2013_SA_STATE_SERVICE"

#endregion

#region machinenames   ##Can be Edited
$spfe01 = "spfe-server0001"
$spfe02 = "spfe-server0002"
$spap01 = "spap-server0001"
$spap02 = "spap-server0002"
#endregion


#region Create_Services_ApplicationPool

################################
# CREATE NEW SERVICES APP POOL #
################################


# Create Managed Accounts and Application Pools 

 
$saAppPoolAccount = Get-SPManagedAccount -Identity $sharedAppPoolAcct
$saAppPool = New-SPServiceApplicationPool -Name $sharedAppPoolName -Account $saAppPoolAccount 
#$saAppPool = Get-SPServiceApplicationPool -Identity $sharedAppPoolName 
#endregion

#region Begin _Provisioning_Service_Applications

#################
# STATE SERVICE #
#################

$stateDB = New-SPStateServiceDatabase -Name $stateServiceSvcDatabaseName
$stateApp = New-SPStateServiceApplication -Name $stateServiceSvcDisplayName -Database $stateDB 
$stateProxy = New-SPStateServiceApplicationProxy -ServiceApplication $stateApp -DefaultProxyGroup 

#################
# USER PROFILES #
#################
 
 $profileApp = New-SPProfileServiceApplication -Name $profileSvcDisplayName `
              -ApplicationPool $saAppPool `
              -ProfileDBName $upaProfileDBName `
              -ProfileDBServer $sharedDatabaseServerName `
              -SocialDBName $upaSocialDBName `
              -SocialDBServer $sharedDatabaseServerName `
              -ProfileSyncDBName $upaSyncDBName `
              -ProfileSyncDBServer $sharedDatabaseServerName
 $profileProxy = New-SPProfileServiceApplicationProxy -ServiceApplication $profileApp

 # Deliberately getting instance of the service on each farm member whether needed or no
 $upasvc1 = Get-SPServiceInstance | ?{($_.typename -match "User Profile Service") -AND ($_.server -match $spfe01)}
 $upasvc2 = Get-SPServiceInstance | ?{($_.typename -match "User Profile Service") -AND ($_.server -match $spfe02)}
 $upasvc3 = Get-SPServiceInstance | ?{($_.typename -match "User Profile Service") -AND ($_.server -match $spap01)}
 $upasvc4 = Get-SPServiceInstance | ?{($_.typename -match "User Profile Service") -AND ($_.server -match $spap02)}
 
 # Start the service where we want it - UPA on All 4
 Start-SPServiceInstance $upasvc1
 Start-SPServiceInstance $upasvc2
 Start-SPServiceInstance $upasvc3
 Start-SPServiceInstance $upasvc4


 ####################
 # MANAGED METADATA #
 ####################

 $mmsApp = New-SPMetadataServiceApplication -Name $mmSvcDisplayName `
            -ApplicationPool $saAppPool  `
            -databasename $mmSvcDatabaseName `
            -DatabaseServer $sharedDatabaseServerName
 $mmsproxy = New-SPMetadataServiceApplicationProxy -Name "Managed Metadata Service Application Proxy" -ServiceApplication $mmsApp

 # Deliberately getting instance of the service on each farm member whether needed or no
 $mmssvc1 = Get-SPServiceInstance | ?{($_.typename -match "Managed Metadata Web Service") -AND ($_.server -match $spfe01)}
 $mmssvc2 = Get-SPServiceInstance | ?{($_.typename -match "Managed Metadata Web Service") -AND ($_.server -match $spfe02)}
 $mmssvc3 = Get-SPServiceInstance | ?{($_.typename -match "Managed Metadata Web Service") -AND ($_.server -match $spap01)}
 $mmssvc4 = Get-SPServiceInstance | ?{($_.typename -match "Managed Metadata Web Service") -AND ($_.server -match $spap02)}
 
 # Start the service where we want it - MMS on All 4
Start-SpServiceInstance $mmssvc1
Start-SpServiceInstance $mmssvc2
Start-SpServiceInstance $mmssvc3
Start-SpServiceInstance $mmssvc4

 ################
 # SECURE STORE #
 ################

 $secStoreApp = New-SPSecureStoreServiceApplication -Name $secureStoreSvcDisplayName `
            -ApplicationPool $saAppPool  `
            -AuditingEnabled `
            -DatabaseName $secureStoreSvcDatabaseName `
            -DatabaseServer $sharedDatabaseServerName
 $secStoreProxy = New-SPSecureStoreServiceApplicationProxy -ServiceApplication $secStoreApp

  # Deliberately getting instance of the service on each farm member whether needed or no
 $sssvc1 = Get-SPServiceInstance | ?{($_.typename -match "Secure Store Service") -AND ($_.server -match $spfe01)}
 $sssvc2 = Get-SPServiceInstance | ?{($_.typename -match "Secure Store Service") -AND ($_.server -match $spfe02)}
 $sssvc3 = Get-SPServiceInstance | ?{($_.typename -match "Secure Store Service") -AND ($_.server -match $spap01)}
 $sssvc4 = Get-SPServiceInstance | ?{($_.typename -match "Secure Store Service") -AND ($_.server -match $spap02)}
 
 # Start the service where we want it - Secure Store on All 4
Start-SpServiceInstance $sssvc1
Start-SpServiceInstance $sssvc2
Start-SpServiceInstance $sssvc3
Start-SpServiceInstance $sssvc4

 
################
# USAGE APP    #
################

$usageSvcInstance = Get-SPUsageService

$usageApp = New-SPUsageApplication `
            -Name           $usageHealthSvcDisplayName `
            -DatabaseServer $sharedDatabaseServerName `
            -DatabaseName   $usageHealthSvcDatabaseName `
            -UsageService   $usageSvcInstance `
            
        $usageHealthSvcProxy = Get-SPServiceApplicationProxy | where {$_.TypeName -like "Usage*"} 
        $usageHealthSvcProxy.Provision()

# End of Service Application Provisioning #
#endregion

#region Deploy_Enterprise_Search_Service


# Using Same Application Pool as the other services

$searchSvcDisplayName = "Search Service Application"

# search admin database settings

$searchAdminDatabaseName   = "CPS_SP2013_SA_SEARCH_ADMIN"


#########################
# SEARCH INITIAL LAYOUT #
#########################
 
# START SERVICES COMPONENTS
$hostA = Get-SPEnterpriseSearchServiceInstance -Identity $spap01
$hostB = Get-SPEnterpriseSearchServiceInstance -Identity $spap02

Start-SPEnterpriseSearchServiceInstance -Identity $hostA
Start-SPEnterpriseSearchServiceInstance -Identity $hostB

#Start the Query Processors - deployed to all machines
Start-SPEnterpriseSearchQueryAndSiteSettingsServiceInstance -Identity $spfe01
Start-SPEnterpriseSearchQueryAndSiteSettingsServiceInstance -Identity $spfe02
Start-SPEnterpriseSearchQueryAndSiteSettingsServiceInstance -Identity $spap01
Start-SPEnterpriseSearchQueryAndSiteSettingsServiceInstance -Identity $spap02

#Wait 2 minutes for the search instances to start
start-sleep -s 180

#PROVISION SERVICE APP AND PROXY

 $searchsa = New-SPEnterpriseSearchServiceApplication -Name $searchSvcDisplayName -ApplicationPool $saAppPool -DatabaseName $searchAdminDatabaseName -DatabaseServer $sharedDatabaseServerName
 $searchsa = Get-SPEnterpriseSearchServiceApplication
 $searchsaproxy = New-SPEnterpriseSearchServiceApplicationProxy -Name "Search Service Application Proxy" -SearchApplication $searchsa

# THIS GIVES US AN EMPTY TOPOLOGY #

# NEED TO ADD THE COMPONENTS TO CREATE A STARTING TOPOLOGY #

$searchsa = get-spenterprisesearchserviceapplication

$newTopology = New-SPEnterpriseSearchTopology -SearchApplication $searchsa

New-SPEnterpriseSearchAdminComponent -SearchTopology $newTopology -SearchServiceInstance $hostA
New-SPEnterpriseSearchAdminComponent -SearchTopology $newTopology -SearchServiceInstance $hostB
New-SPEnterpriseSearchCrawlComponent -SearchTopology $newTopology -SearchServiceInstance $hostA
New-SPEnterpriseSearchCrawlComponent -SearchTopology $newTopology -SearchServiceInstance $hostB
New-SPEnterpriseSearchAnalyticsProcessingComponent -SearchTopology $newTopology -SearchServiceInstance $hostA
New-SPEnterpriseSearchAnalyticsProcessingComponent -SearchTopology $newTopology -SearchServiceInstance $hostB
New-SPEnterpriseSearchContentProcessingComponent -SearchTopology $newTopology -SearchServiceInstance $hostA
New-SPEnterpriseSearchContentProcessingComponent -SearchTopology $newTopology -SearchServiceInstance $hostB
New-SPEnterpriseSearchQueryProcessingComponent -SearchTopology $newTopology -SearchServiceInstance $hostA
New-SPEnterpriseSearchQueryProcessingComponent -SearchTopology $newTopology -SearchServiceInstance $hostB
New-SPEnterpriseSearchIndexComponent -SearchTopology $newTopology -SearchServiceInstance $hostA -IndexPartition 0
New-SPEnterpriseSearchIndexComponent -SearchTopology $newTopology -SearchServiceInstance $hostB -IndexPartition 0

# Need to set the Search Admin component to be able to activate a new topology for the first time
Set-SPEnterpriseSearchAdministrationComponent -SearchApplication $searchsa -SearchServiceInstance $hostA

#Wait 2 minutes for the search instances to set the administration component.
start-sleep -s 180

# ACTIVATE NEW TOPOLOGY #   # This may take about 25 minutes

Write-Host "This may take about 25 minutes."
Set-SPEnterpriseSearchTopology -Identity $newTopology

Get-SPEnterpriseSearchTopology -SearchApplication $searchsa
Get-SPEnterpriseSearchStatus -SearchApplication $searchsa -Text

#endregion
