﻿
Add-PSSnapin Microsoft.SharePoint.PowerShell -ErrorAction SilentlyContinue
Import-Module .\SP2013_GlobalFunctions.psm1 -AsCustomObject -Force -DisableNameChecking

#This script needs to be executed on each server being joined to the farm

#region Core_Farm_Variables

# Existing Database Information
$configDatabaseServer = "SQLCL01-AG01-L01"  #Can be Edited
$configDatabaseName   = "CPS_SP2013_CONFIG" #Can be Edited 

# Farm Passphrase
$farmPassphrase = 'LS1setup!' # use single quotes here avoid issues with having a $ in the password

#endregion


#region Start_Farm_Add

$secureFarmPassphrase = ConvertTo-SecureString $farmPassphrase -AsPlainText -Force

Log-Message -Message "`nJoining Server to Existing Farm"
Connect-SPConfigurationDatabase -DatabaseName $configDatabaseName -DatabaseServer $configDatabaseServer -Passphrase $secureFarmPassphrase      
Log-Message -Message "Server has successfully been joined to farm`n"

if(!$?) {return}

Remove-PSSnapin Microsoft.SharePoint.PowerShell
Add-PSSnapin Microsoft.SharePoint.PowerShell -ErrorAction SilentlyContinue | Out-Null
 
# Installs the Help site collection files for SharePoint 2013 Products in the current farm.
Log-Message -Message "`tPerforming Configuration task 1 of 5 -  Installing help collections..."   
Install-SPHelpCollection -All

# Enforces resource security on the local server
Log-Message -Message "`tPerforming Configuration task 2 of 5 -  Securing the SharePoint resources..."   
Initialize-SPResourceSecurity

# Installs and optionally provisions services on a farm
Log-Message -Message "`tPerforming Configuration task 3 of 5 -  Registering SharePoint services..."   
Install-SPService

# Installs all available features.
Log-Message -Message "`tPerforming Configuration task 4 of 5 -  Registering SharePoint features..."   
Install-SPFeature -AllExistingFeatures | Out-Null

# Copies shared application data to existing Web application folders.
Log-Message -Message "`tPerforming Configuration task 5 of 5 -  Installing the application content files...`n"   
Install-SPApplicationContent

# start the OWSTIMER service
Stop-Service  SPTimerV4
Start-Service SPTimerv4

#endregion