﻿

$webAppName = "Sharepoint Sites"     # Specifies the display name of the new Web application.
$publicUrl  = "https://www.sprich.com"       # Can be Edited: Specifies the load-balanced URL for the Web application  #Vijay-Updated 

# application pool settings
$webAppAppPoolName = "SharePoint Content"         # Specifies the display name of the application pool in IIS
$webAppAppPoolUser = "richsp\svc_spcontent"   # Specifies the user account that this application pool will run as   # Vijay-Updated


# authentication settings 
$allowAnonymousAccess = $false                # Allows anonymous access to the Web application
$authenticationMethod = "NTLM"                # NTLM or Kerberos are the two value values, Not used if $useWindowsClaims is True
$useWindowsClaims     = $true                 # Set this to true to create a Windows Claims Web Application, which is the recomended setting in SharePoint 2013.

# database settings
$databaseServer = "SQLCL01-AG01-L01" # Specifies the database server name. The format can be server or server\instance  #Vijay-Updated 
$databaseName   = "CPS_SP2013_CONTENT_001" # Specifies the name of the initial content database for the new Web application.

# IIS settings

    # Specifies a valid URL assigned to the Web application that must correlate to the alternate access mapping configuration, in the form server_name.
    # When the HostHeader parameter is present, the value of this field is the internal URL for the Web application. 
    # Leave this empty if creating HH sites in the web app
    $hostHeader = "www.sprich.com" #Vijay-Updated 
    $port       = 443                          # Specifies the port on which this Web application can be accessed. 
    $useSSL     = $true                       # Enables Secure Sockets Layer (SSL) encryption for this Web application. 
    $path       = "C:\inetpub\wwwroot\wss\VirtualDirectories\sprich.com"  #Can be Edited

# additional wild card managed paths to add to the web application
$additionalManagedPaths = @("personal")




<############    YOU SHOULD NOT HAVE TO MODIFY ANYTHING BELOW THIS POINT    ############>
Import-Module .\SP2013_GlobalFunctions.psm1 -AsCustomObject -Force -DisableNameChecking
Add-PSSnapin Microsoft.SharePoint.PowerShell -ErrorAction SilentlyContinue | Out-Null

$stopWatch = New-Object System.Diagnostics.Stopwatch
$stopWatch.Start()


Log-Message -Message "`nSTEP 1/3: Validating Content Database Name"

    $contentDB = Get-SPContentDatabase $databaseName -ErrorAction SilentlyContinue
    if($contentDB)
    {
        Log-Message -Message "A content database with the name '$databaseName' already exists in the farm.  Please choose a unique name.`n" -Color "Red"
        return
    }



Log-Message -Message "`nSTEP 2/3: Provisioning Application Pool"


    $applicationPool= New-ApplicationAppPool -Name $webAppAppPoolName -Account $webAppAppPoolUser



Log-Message -Message "`nSTEP 3/3: Provisioning Web Application"

    $params = @{
        Name                   = $webAppName;
        ApplicationPool        = "$($applicationPool.Name)";
        ApplicationPoolAccount = "$($applicationPool.ProcessAccountName)";
        AllowAnonymousAccess   = $allowAnonymousAccess;
        AuthenticationMethod   = $authenticationMethod;
        DatabaseName           = $databaseName;
        DatabaseServer         = $databaseServer;
        HostHeader             = $hostHeader;
        Url                    = $publicUrl;
        Path                   = $path;
        Port                   = $port;
        SecureSocketsLayer     = $useSSL;
    }

    if($useWindowsClaims) 
    { 
        $params["AuthenticationProvider"] = $(New-SPAuthenticationProvider) 
    }

    # create the web application
    $webApp = New-SPWebApplication @params -WarningAction SilentlyContinue

    if(!$?)
    {
        return
    }

    # The web app URL that sharepoint will use will have an trailing /
    $publicUrl = $publicUrl.TrimEnd('/')
    $publicUrl = $publicUrl + "/"

    # Get the new web application object to to avoid an update conflict with OWSTIMER when saving the value below
    Sleep -Seconds 5 # let owstimer complete on the remaining servers in the farm.
    $webApp = Get-SPWebApplication | WHERE { $_.Url -eq $publicUrl }

    if(!$webApp)
    {
        Log-Message -Message "Failed to get a web application object with the URL: $publicUrl" -Color "Red"
        return
    }


    Log-Message -Message "`Creating First default teamsite"

    $thesite = New-SPSite -Url $publicUrl -Template STS#0 -Description "Home" -OwnerAlias "richsp\spAdmin"    #Can be edited

    Log-Message -Message "You need to assign an SSL certificate to the web application before browsing to the site"

$stopWatch.Stop()
$ts = $stopWatch.Elapsed

Log-Message -Message ""
Log-Message -Message $("`nScript Completed in {0} minutes, {1} seconds" -f $ts.Minutes, $ts.Seconds )
