<#

 Microsoft provides programming examples for illustration only, without warranty either expressed or
 implied, including, but not limited to, the implied warranties of merchantability and/or fitness 
 for a particular purpose. 
 
 This sample assumes that you are familiar with the programming language being demonstrated and the 
 tools used to create and debug procedures. Microsoft support professionals can help explain the 
 functionality of a particular procedure, but they will not modify these examples to provide added 
 functionality or construct procedures to meet your specific needs. if you have limited programming 
 experience, you may want to contact a Microsoft Certified Partner or the Microsoft fee-based consulting 
 line at (800) 936-5200. 


 ----------------------------------------------------------
 History
 ----------------------------------------------------------
 03-27-2013 - Created
 05-23-2013 - Added Test-Credential
 05-23-2013 - Updated proxy code in the Download-Package method to workaround a bug with the credential object being reset.
 07-15-2013 - Updated proxy code to test for null
 07-26-2013 - Updated to fix a regression in proxy credential getting corrupted in calling script.
 09-03-2013 - Added exit code 17025 == Patch already installed
 03-10-2014 - Replaced tabs with spaces for better readablity with notepad
 04-22-2014 - Removed HostHeaderSiteConfiguration object after simplifying SP2013_11_CreateHostHeaderSiteCollections.ps1
 04-29-2014 - Added Test-DirectoryLocationEmpty for search script
 05-29-2014 - Updated function Test-Credential
 10-06-2014 - Updated Test-DirectoryLocationEmpty

#==============================================================


#>


$verboseOutput = $true

function Test-DirectoryLocationEmpty([string]$Path)
{
    if(!(Test-Path -Path $Path))
    {
        return $false
    }

    $fileCount = [System.IO.Directory]::GetFiles($Path).Length
    if($fileCount -gt 0)
    {
        return $false
    }

    # make sure all the sub directories do not contain files
    $directories = [System.IO.Directory]::GetDirectories($Path)
    
    foreach($dir in $directories)
    {
        if(-not (Test-DirectoryLocationEmpty -Path $dir))
        {
            return $false
        }   
    }
   
    return $true
}

function Log-Message([string]$Message, [string]$Color = "White", [bool]$NoNewLine = $false) 
{
    if($verboseOutput -or $Color -eq "Red")
    {
        Write-Host $Message -ForegroundColor $Color -NoNewline:$NoNewLine
    }
}

function New-CompleteFlagFile
{
    New-Item $env:windir\temp\284a609f-3e06-4c47-9b82-f99185fc24c4.txt -Type File | Out-Null
}

function Get-CompleteFlagFile
{
    return Get-Item $env:windir\temp\284a609f-3e06-4c47-9b82-f99185fc24c4.txt -ErrorAction SilentlyContinue
}

function New-ContinueFlagFile
{
    New-Item $env:windir\temp\fb4c861e-b31d-4505-aa21-935cba08264e.txt -Type File | Out-Null
}

function Get-ContinueFlagFile
{
    return Get-Item $env:windir\temp\fb4c861e-b31d-4505-aa21-935cba08264e.txt -ErrorAction SilentlyContinue
}

function Remove-ContinueFlagFile
{
    Remove-Item $env:windir\temp\fb4c861e-b31d-4505-aa21-935cba08264e.txt -Force -Confirm:$false
}

function Install-SharePoint2013Prerequisites([string]$InstallerPath, [string]$PrerequisiteInstallerArguments)
{
    if(Get-CompleteFlagFile -ne $null)
    {
        Log-Message -Message "SharePoint 2013 Prerequisites have been installed successfully`n"
	    return 0
    }

    if(Get-ContinueFlagFile -ne $null)
    {
        Log-Message -Message "Resuming Prerequisite Installation"
        $PrerequisiteInstallerArguments = "/continue " + $PrerequisiteInstallerArguments
        Remove-ContinueFlagFile
    }
    else
    {
        Log-Message -Message "Starting Prerequisite Installation"
    }
    
    if(-not(Test-Path -LiteralPath $InstallerPath -PathType Leaf))
    {
        Log-Message -Message "PrerequisiteInstaller.exe not found.`n" -Color "Red"  
        return $returnCode
    }

    $process  = Start-Process -Wait -PassThru $InstallerPath $PrerequisiteInstallerArguments
    $exitCode = Check-ProcessExitCode -ExitCode $process.ExitCode
    
    switch($exitCode)
    {
        0 {
            # setting a simple txt file as a flag to indicate that all the preqrequisites have been installed
            New-CompleteFlagFile
        }
        1001 {
            # setting a simple txt file as a flag to indicate that the preqrequisite installer needs the /continue flag
            New-ContinueFlagFile
        }
        3010 {
            # setting a simple txt file as a flag to indicate that the preqrequisite installer needs the /continue flag
            New-ContinueFlagFile
        }
    }
    
    return $exitCode
}


function Check-ProcessExitCode([int]$ExitCode)
{
    $code = $ExitCode

    switch($ExitCode)
    {
        -2145124329 {
            Log-Message -Message "`tResult:     Not applicable to this operating system" -Color "Yellow"
            $code = 0
        }
        -2146869246 {
            Log-Message -Message "`tResult:     Not applicable to this operating system" -Color "Yellow"
            $code = 0
        }
        0 {
            Log-Message -Message "`tResult:     Installation was successful" -Color "Green"
            $code = 0
        }
        2 {
            Log-Message -Message "`tResult:     An invalid argument was provided to the Prerequisite Installer.`n" -Color "Red"
        }
        776 {
            Log-Message -Message "`tResult:     Package will not install due to a pending reboot.`n" -Color "Red"
        }
        1001 {
            Log-Message -Message "`nA reboot is required to continue the installation. Please reboot the server and re-execute this script to continue the installation.`n" -Color "Yellow"
        }
        1605 {
            Log-Message -Message "`tResult:     Unknown Product" -Color "Red"
            $code = 0
        }
        1618 {
            Log-Message -Message "`tResult:     Another installation is already in progress.  Complete that installation before proceeding with this install." -Color "Yellow"
        }
        1641 {
            Log-Message -Message "`tResult:     Installation was successful, a reboot is required to complete intallation" -Color "Yellow"
        }
        1642 {
            Log-Message -Message "`tResult:     Not applicable to this operating system" -Color "Yellow"
            $code = 0
        }
        3010 {
            Log-Message -Message "`nA reboot is required to continue the installation. Please reboot the server and re-execute this script to continue the installation.`n" -Color "Yellow"
        }
        17025 {
            Log-Message -Message "`tResult:     Update has already been installed." -Color "Yellow"
            $code = 0
        }
        2359302 {
            Log-Message -Message "`tResult:     Update has already been installed." -Color "Yellow"
            $code = 0
        }
        default {
            Log-Message -Message "`tResult:     An unexpected error code was returned: $ExitCode" -Color "Red"
        }
    }

    return $code
}


function Download-Packages([string]$DestinationPath, [System.Collections.Hashtable]$Packages, [string]$PackageDescription="Unknown", [System.Management.Automation.PSCredential]$ProxyCredential=[System.Management.Automation.PSCredential]::Empty, [string]$ProxyAuthType="NTLM")
{
    Log-Message -Message "Downloading $($PackageDescription):"

    # enumerate each of the packages
    foreach($package in $Packages.GetEnumerator())
    {
        try
        {
            # grab the url value
            $packageUrl  = $package.Value
            # grab the file download name
            $packageName = $packageUrl.Split('/')[-1]
            # build the full path to the download location
            $packagePath = [System.IO.Path]::Combine($DestinationPath, $packageName)

            # if the file doesn't already exists, download it
            if(-Not(Test-Path -LiteralPath $packagePath -PathType Leaf ))
            {
                Log-Message -Message "`tDownloading: $packageName"

                if($ProxyCredential -and $ProxyCredential -ne [System.Management.Automation.PSCredential]::Empty)
                {
                    # Create a instance copy of the username. The default is a ref pointer and the value is wiped out after a call to Start-BitsTransfer, taking our pointer value along with it.
                    $userName  = [string]::Copy($ProxyCredential.UserName)
                    $securePwd = $ProxyCredential.Password

                    # bug in PowerShell v3 on Win7 will reset the ProxyCredential UserName property after a call to Start-BitsTransfer, 
                    # so we need to create a new PSCredential object (not a ref copy of the original object or it will get corrupted in the caller script)
                    $tempProxyCred = New-Object System.Management.Automation.PSCredential($userName, $securePwd)

                    Log-Message -Message "`tDownloading with Proxy user Credential:$($tempProxyCred.UserName)"
                    Start-BitsTransfer -Source $packageUrl -Destination $DestinationPath -DisplayName "Downloading $packageName" -Priority Foreground -ProxyAuthentication $ProxyAuthType -ProxyCredential $tempProxyCred
                }
                else
                {
                    Start-BitsTransfer -Source $packageUrl -Destination $DestinationPath -DisplayName "Downloading $packageName" -Priority Foreground
                }
            }
            else
            {
                Log-Message -Message "`tSkipping:    $packageName" -Color "Yellow"
            }
        }
        catch
        {
            Log-Message -Message "Error downloading $packageName" -Color "Red"
        }
    }
    
    Log-Message -Message "Downloading of $PackageDescription Complete`n"
}


function Create-SubDirectories([string]$RootPath, [object[]]$SubDirectories)
{
    Log-Message -Message "Creating Required Directory Structure"
    $result = $true
    foreach($folder in $SubDirectories)
    {
        try
        {
            $path = $([System.IO.Path]::Combine($RootPath, $folder))
		
            if(!(Test-Path -LiteralPath $path -PathType Container))
            {
                Log-Message -Message "`tCreating: $path"
                New-Item $path -Type Directory | Out-Null
            }
        }
        catch
        {
            Log-Message $?
            $result = $false
        }
    }
    Log-Message -Message "Creation of Required Directory Structure Complete`n"
	
    return $result
}


function Extract-Packages([string]$DestinationPath, [System.Collections.Hashtable]$Packages, [string]$PackageDescription="Unknown")
{
	Log-Message -Message "Extracting $PackageDescription"

    foreach($package in $Packages.GetEnumerator())
    {
        $downloadedName = ($package.Value).Split('/')[-1]
        $targetName     = $package.Name
        $currentPath    = [System.IO.Path]::Combine($DestinationPath, $downloadedName)
        $targetPath     = [System.IO.Path]::Combine($DestinationPath, $targetName)
        
        if(-Not(Test-Path -LiteralPath $targetPath -PathType Leaf ))
        {
            Log-Message -Message "`tExtracting: $downloadedName"
    
            if(-Not(Test-Path -LiteralPath $currentPath -PathType Leaf ))
            {
                Log-Message -Message "`tError:   Could not find file $currentPath." -Color "Red"
                continue
            }

            try
            {
                Log-Message -Message "`tRenaming:   $downloadedName"
                Rename-Item -Path $currentPath -NewName $("$downloadedName.zip") -Force
                $currentPath += ".zip"
		    }
            catch
            {
                Log-Message -Message "Failed to rename $downloadedName" -Color "Red"
		        continue
            }
		    
            try
            {
                # extract the package
                $shell = New-Object -com Shell.Application

                # get a shell object for the destination folder and zip package
                $folder  = $shell.NameSpace($DestinationPath)
                $zipfile = $shell.NameSpace($currentPath)
		        
                # extract
                $folder.CopyHere($zipfile.Items(), 16 <# Respond with "Yes to All" for any dialog box that is displayed. #>)
            }
            catch
            {
                Write-Host "Error extracting $downloadedName" -ForegroundColor Red
                return
            }

            try
            {
                Log-Message -Message "`tDeleting:   $downloadedName.zip"
                Remove-Item -Path $currentPath -Confirm:$false -Force
            }
            catch
            {
                Log-Message "Error deleting $downloadedName.zip" -Color "Red"
                continue
            }
        }
        else
        {
            Log-Message -Message "`tSkipping:    $downloadedName" -Color "Yellow"
        }
    }

    $hotfixTxt = [System.IO.Path]::Combine($DestinationPath, "hotfix.txt")
    Remove-Item -Path $hotfixTxt -Confirm:$false -Force -ErrorAction SilentlyContinue
	
    Log-Message -Message "Extraction of $PackageDescription Complete`n"
}


function Check-IsSystemRebootPending
{
    $updateExeVolatile = Get-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Updates\UpdateExeVolatile\SOFTWARE\Microsoft\Updates" -Name UpdateExeVolatile -ErrorAction SilentlyContinue
    if($updateExeVolatile -ne $null -and $updateExeVolatile -ne 0) { return $true }

    $pendingFileRenameOperations = Get-ItemProperty -Path "HKLM:\SYSTEM\CurrentControlSet\Control\Session Manager" -Name PendingFileRenameOperations -ErrorAction SilentlyContinue
    if($pendingFileRenameOperations) { return $true }

    return Test-Path -Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\WindowsUpdate\Auto Update\RebootRequired"
}


function New-ApplicationAppPool([string]$Name, [string]$Account)
{
    $appPool = Get-SPServiceApplicationPool -Identity $Name -ErrorAction SilentlyContinue
    
    if($appPool -eq $null -or $appPool.ProcessAccountName -ne $Account )
    {
        $managedAccount = Get-SPManagedAccount -Identity $Account -ErrorAction SilentlyContinue
    
        if($managedAccount -eq $null)
        {
            
            $credential = Get-Credential -Message "Please enter the password for account: $Account" -UserName $Account -ErrorAction SilentlyContinue
            
            if($credential)
            {
                $managedAccount = New-SPManagedAccount -Credential $credential
            }
            else
            {
                Log-Message -Message "Failed to obtain valid credentials for $Account." -Color "Red"
                exit -1
            }
        }

        $managedAccount = Get-SPManagedAccount -Identity $Account -ErrorAction SilentlyContinue

        if($managedAccount -eq $null)
        {
            Log-Message -Message "Unable to create the managed account in SharePoint." -Color "Red"
            exit -1
        }

        $appPool = New-SPServiceApplicationPool -Name $Name -Account $managedAccount
    }

    return $appPool 
}

# only reference this is the 3.5 framework is installed, meaning we have installed SharePoint 2013 prereqs already
if((Test-Path "HKLM:\Software\Microsoft\NET Framework Setup\NDP\v3.5") -and (Get-ItemProperty "HKLM:\Software\Microsoft\NET Framework Setup\NDP\v3.5").Install -ne $null)
{
$QuotaTemplate = @"
        using System;
        namespace SharePoint.Install {
            public class QuotaTemplate 
            {
                public String  Name                 {get; set;}
                public long    StorageMaximumLevel  {get; set;}
                public long    StorageWarningLevel  {get; set;}
                public double  UserCodeMaximumLevel {get; set;}
                public double  UserCodeWarningLevel {get; set;}
            
                public QuotaTemplate(string name, long storageMaximumLevel, long storageWarningLevel, double userCodeMaximumLevel, double userCodeWarningLevel)
                {
                    this.Name = name; 
                    this.StorageMaximumLevel  = storageMaximumLevel; 
                    this.StorageWarningLevel  = storageWarningLevel;
                    this.UserCodeMaximumLevel = userCodeMaximumLevel;
                    this.UserCodeWarningLevel = userCodeWarningLevel;
                }
            }
        }
"@
    Add-Type $QuotaTemplate -Language CSharpVersion3 -ErrorAction SilentlyContinue 
}



function Test-Credential([PSCredential]$Credential)
{
    if(!$Credential){ return $false }

    Add-Type -AssemblyName System.DirectoryServices.AccountManagement

    $ct = [System.DirectoryServices.AccountManagement.ContextType]::Domain
    $pc = New-Object System.DirectoryServices.AccountManagement.PrincipalContext($ct, $($Credential.GetNetworkCredential().Domain))

    return $pc.ValidateCredentials($($Credential.GetNetworkCredential().UserName),$($Credential.GetNetworkCredential().Password))
}
