﻿# SP1 schema version 15292
# CU3 schema version 15283
# CU2 schema version 15281
# RTM schema version 15137

Import-Module $PSScriptRoot\..\..\xPDT.psm1

Function Get-TargetResource
{
    [CmdletBinding()]
    [OutputType([System.Collections.Hashtable])]
    param(
    [Parameter(Mandatory=$true)]
    [string]$SchemaVersion,

    [Parameter(Mandatory=$true)]
    [string]$Organization,

    [Parameter(Mandatory=$true)]
    [String]
    $SourcePath,

    [String]
    $SourceFolder,

    [Parameter(Mandatory=$true)]
    [PSCredential]
    $Credential
     )

    $ADRootDSE = (Get-ADRootDSE -ErrorAction Stop).NamingContexts | Where-Object {$_ -match 'CN=Schema,CN=Configuration,*'}
    try
    {
        Write-Verbose "Checking for objects within $ADRootDSE"
        $CurrentSchemaVersion = (Get-ADObject "CN=ms-Exch-Schema-Version-Pt,$ADRootDSE" -Property rangeUpper).rangeUpper
    }
    
    catch [Microsoft.ActiveDirectory.Management.ADIdentityNotFoundException]
    {
        write-verbose 'Exchange schema not found'
    }
    catch
    {
        if ($error[0]) {Write-Verbose $error[0].Exception}
    }

    return @{Ensure = $Ensure; SchemaVersion = $CurrentSchemaVersion}
}

Function Set-TargetResource
{
    [CmdletBinding()]
    param(
    [Parameter(Mandatory=$true)]
    [string]$SchemaVersion,

    [Parameter(Mandatory=$true)]
    [string]$Organization,

    [Parameter(Mandatory=$true)]
    [String]
    $SourcePath,

    [String]
    $SourceFolder,

    [Parameter(Mandatory=$true)]
    [PSCredential]
    $Credential
     )

    try 
    {
        $Path = Join-Path -Path (Join-Path -Path $SourcePath -ChildPath $SourceFolder) -ChildPath 'setup.exe'
        $Path = ResolvePath $Path
        Write-Verbose "Path: $Path"

        $Arguments = "/PrepareAD /on:$Organization /IAcceptExchangeServerLicenseTerms"
        Write-Verbose "Arguments: $Arguments"

  	    $Process = StartWin32Process -Path $Path -Arguments $Arguments -Credential $Credential -AsTask
	    Write-Verbose $Process
	    WaitForWin32ProcessEnd -Path $Path -Arguments $Arguments -Credential $Credential

        $global:DSCMachineStatus=1
    }
    catch
    {
        if ($error[0]) {Write-Verbose $error[0].Exception}
        throw 'An error occured while running the Exchange Setup command to extend the schema.'
    }


}

Function Test-TargetResource
{
    [CmdletBinding()]
    [OutputType([System.Boolean])]
    param(
    [Parameter(Mandatory=$true)]
    [string]$SchemaVersion,

    [Parameter(Mandatory=$true)]
    [string]$Organization,

    [Parameter(Mandatory=$true)]
    [String]
    $SourcePath,

    [String]
    $SourceFolder,

    [Parameter(Mandatory=$true)]
    [PSCredential]
    $Credential
     )
    
    Write-Verbose "Testing if Exchange schema version $schemaversion is present."
    try
    {
        $Get = Get-TargetResource @PSBoundParameters
        return ($SchemaVersion -eq $Get.SchemaVersion)
    }
    catch
    {
        if ($error[0]) {Write-Verbose $error[0].Exception}
        return $false
    }
}

Export-ModuleMember -Function *-TargetResource