Import-LocalizedData LocalizedData -filename MSFT_xExchangeSetup.strings.psd1
Import-Module $PSScriptRoot\..\..\xPDT.psm1

#region GET
function Get-TargetResource 
{
    [CmdletBinding()]
    [OutputType([System.Collections.Hashtable])]
     param
    (
        [Parameter(Mandatory=$true)]
        [String]
        $SourcePath,

        [String]
        $SourceFolder,
        
        [Parameter(Mandatory=$true)]
        [System.String]
        $Arguments,
        
        [Parameter(Mandatory=$true)]
        [PSCredential]
        $Credential
    )
        
    $Path = Join-Path -Path (Join-Path -Path $SourcePath -ChildPath $SourceFolder) -ChildPath "setup.exe"
    $Path = ResolvePath $Path
    $Version = (Get-Item -Path $Path).VersionInfo.ProductVersion

    switch($Version)
    {
        '15.00.0847.030'
        {
            $IdentifyingNumber = '{4934D1EA-BE46-48B1-8847-F1AF20E892C1}'
        }
        Default
        {
            throw $LocalizedData.UnknownVersion
        }
    }
        
    return @{
		SourcePath = $SourcePath
        SourceFolder = $SourceFolder
		Arguments = $Arguements
		Credential = $Credential
		Version = (Get-WmiObject -Class Win32_Product | Where-Object {$_.IdentifyingNumber -eq $IdentifyingNumber}).Version
		}
}
# Get-TargetResource -SourcePath 'F:\Setup.exe' -Arguements '/mode:install' -Credential (get-credential)
# Expectation is the function will return a hashtable with a boolean for Result to indicate whether the application is install and information about how the application was installed
#endregion

#region SET
function Set-TargetResource
{
    [CmdletBinding()]
     param
    (
        [Parameter(Mandatory=$true)]
        [String]
        $SourcePath,

        [String]
        $SourceFolder,
        
        [Parameter(Mandatory=$true)]
        [System.String]
        $Arguments,
        
        [Parameter(Mandatory=$true)]
        [PSCredential]
        $Credential
    )

    $Path = Join-Path -Path (Join-Path -Path $SourcePath -ChildPath $SourceFolder) -ChildPath "setup.exe"
    $Path = ResolvePath $Path
    Write-Verbose "Path: $Path"

    Write-Verbose "Arguments: $Arguments"
	
	$Process = StartWin32Process -Path $Path -Arguments $Arguments -Credential $Credential -AsTask
	Write-Verbose $Process
	WaitForWin32ProcessEnd -Path $Path -Arguments $Arguments -Credential $Credential

    $global:DSCMachineStatus=1

}
# Set-TargetResource -Path 'F:\Setup.exe' -Arguements '/mode:install' -Credential (get-credential)
# Expectation is the application would be installed silently
#endregion

#region TEST
function Test-TargetResource 
{
    [CmdletBinding()]
    [OutputType([System.Boolean])]
     param
    (
        [Parameter(Mandatory=$true)]
        [String]
        $SourcePath,

        [String]
        $SourceFolder,
        
        [Parameter(Mandatory=$true)]
        [System.String]
        $Arguments,
        
        [Parameter(Mandatory=$true)]
        [PSCredential]
        $Credential
    )

        $Path = Join-Path -Path (Join-Path -Path $SourcePath -ChildPath $SourceFolder) -ChildPath "setup.exe"
        $Path = ResolvePath $Path
        $Version = (Get-Item -Path $Path).VersionInfo.ProductVersion
    
        $Get = Get-TargetResource @PSBoundParameters

        Write-Verbose "Validating version $Version"

        switch($Version)
        {
            '15.00.0847.030'
            {
                return $Get.Version -eq '15.0.847.32'
            }
            Default
            {
                throw $LocalizedData.UnknownVersion
            }
        }

}
# Test-TargetResource -Path 'F:\Setup.exe' -Arguements '/mode:install' -Credential (get-credential)
# Expectation is a boolean response based on whether the application is installed
#endregion

Export-ModuleMember -Function *-TargetResource