﻿# SP1 schema version 15292
# CU3 schema version 15283
# CU2 schema version 15281
# RTM schema version 15137

Function Get-TargetResource
{
    [CmdletBinding()]
    [OutputType([System.Collections.Hashtable])]
    param(
    [Parameter(Mandatory=$true)]
    [string]$SchemaVersion,
    
    [UInt64]$RetryIntervalSec = 10,

    [UInt32]$RetryCount = 720
    )
    
    try
    {
        $CurrentVersionExchange = (Get-ADObject "CN=ms-Exch-Schema-Version-Pt,$((Get-ADRootDSE -ErrorAction SilentlyContinue).NamingContexts | Where-Object {$_ -match 'CN=Schema,CN=Configuration,*'})" -Property rangeUpper -ErrorAction Stop).rangeUpper
    }
    catch
    {
        if ($error[0]) {Write-Verbose $error[0].Exception}
    }

    return @{SchemaVersion = $CurrentVersionExchange}
}

Function Set-TargetResource
{
    [CmdletBinding()]
    param(
    [Parameter(Mandatory=$true)]
    [string]$SchemaVersion,
    
    [UInt64]$RetryIntervalSec = 10,

    [UInt32]$RetryCount = 720
    )
    
    Write-Verbose -Message "Checking for Exchange schema version $SchemaVersion"

    for ($count = 0; $count -lt $RetryCount; $count++)
    {   
        $schema = $SchemaVersion -eq (Get-TargetResource @PSBoundParameters).SchemaVersion     
        if ($schema -eq $true)
        {
            Write-Verbose -Message "The correct version was found"
            break
        }
        else
        {
            Write-Verbose -Message "Correct schema version was NOT found"
            Write-Verbose -Message "Retrying in $RetryIntervalSec seconds ..."
            Start-Sleep -Seconds $RetryIntervalSec
        }
    }

    if (!$schema)
    {
        throw "Schema $schemaversion NOT found after $RetryCount attempts."
    }
}

Function Test-TargetResource
{
    [CmdletBinding()]
    [OutputType([System.Boolean])]
    param(
    [Parameter(Mandatory=$true)]
    [string]$SchemaVersion,
    
    [UInt64]$RetryIntervalSec = 10,

    [UInt32]$RetryCount = 720
    )
    
    $SchemaVersion -eq (Get-TargetResource @PSBoundParameters).SchemaVersion
}

Export-ModuleMember -Function *-TargetResource