@{
    AllNodes = @(
		#Settings in this section will apply to all nodes. For the purposes of this demo,
        #the only thing that will be configured in here is how credentials will be stored
        #in the compiled MOF files.
        @{
            NodeName        = '*'

            ###SECURE CREDENTIAL PASSING METHOD###
            #This is the preferred method for passing credentials, as they are not stored in plain text. See:
            #http://blogs.msdn.com/b/powershell/archive/2014/01/31/want-to-secure-credentials-in-windows-powershell-desired-state-configuration.aspx
            
            # The path to the .cer file containing the 
            # public key of the Encryption Certificate 
            # used to encrypt credentials for this node 
            CertificateFile = 'C:\publickey.cer' 

            # The thumbprint of the Encryption Certificate 
            # used to decrypt the credentials on target node 
            Thumbprint      = '39bef4b2e82599233154465323ebf96a12b60673' 


            ###UNSECURE CREDENTIAL PASSING METHOD###
            #This should NOT go in a production demo. Need to encourage customers to use the secure way to pass creds.
            #PSDscAllowPlainTextPassword = $true 
        }

		#Each individual server that will be configured needs to be defined next.
        @{
            NodeName = 'e15-1'
            CASID    = 'Site1CAS'
        }

        @{
            NodeName = 'e15-2'
            CASID    = 'Site2CAS'
        }
    );

	#CAS settings that are unique per site will go in separate hash table entries.
	Site1CAS = @(
        @{
            InternalNLBFqdn            = 'mail-site1.mikelab.local'
            ExternalNLBFqdn            = 'mail.mikelab.local'

            #ClientAccessServer Settings
            AutoDiscoverSiteScope      = 'Site1','Site3','Site5'

            #OAB Settings
            OABsToDistribute           = 'Default Offline Address Book - Site1'
        }
    );

	Site2CAS = @(
        @{
            InternalNLBFqdn            = 'mail-site2.mikelab.local'
            ExternalNLBFqdn            = 'mail.mikelab.local'

            #ClientAccessServer Settings
            AutoDiscoverSiteScope      = 'Site2','Site4','Site6'

            #OAB Settings
            OABsToDistribute           = 'Default Offline Address Book - Site2'
        }
    );
}