﻿Configuration SC2012R2OM
{
    Import-DscResource -Module xPDT
    
    Node $AllNodes.Where{$_.Role -eq "System Center 2012 R2 Operations Manager Database Server"}.NodeName
    {
        WindowsFeature NET351
        {
            Name = "NET-Framework-Core"
            Ensure = "Present"
            Source = $Node.SourcePath +  "\WindowsServer2012R2\sources\sxs"
        }

        xSQLServer2012Install MSSQLSERVER
        {
            DependsOn = "[WindowsFeature]NET351"
            SourcePath = $Node.SourcePath
            InstanceName = "MSSQLSERVER,FULLTEXT"
            Features = "SQLENGINE"
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
        }
    }

    Node $AllNodes.Where{$_.Role -eq "System Center 2012 R2 Operations Manager Datawarehouse Server"}.NodeName
    {
        WindowsFeature NET351
        {
            Name = "NET-Framework-Core"
            Ensure = "Present"
            Source = $Node.SourcePath +  "\WindowsServer2012R2\sources\sxs"
        }

        xSQLServer2012Install MSSQLSERVER
        {
            DependsOn = "[WindowsFeature]NET351"
            SourcePath = $Node.SourcePath
            InstanceName = "MSSQLSERVER,FULLTEXT"
            Features = "SQLENGINE"
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
        }
    }

    Node $AllNodes.Where{$_.Role -eq "System Center 2012 R2 Operations Manager Management Server"}.NodeName
    {
        Group Admin
        {
            Ensure = "Present"
            GroupName = (Get-WMIObject Win32_Group -filter "LocalAccount=True AND SID='S-1-5-32-544'" -ComputerName $Node.NodeName).Name
            MembersToInclude = $Node.?.UserName
            Credential = $Node.InstallerServiceAccount
        }

        WindowsFeature NET45
        {
            Name = "NET-Framework-45-Core"
            Ensure = "Present"
        }

        WaitForAll SC2012R2OMDB
        {
            NodeName = $AllNodes.Where{$_.Role -eq "System Center 2012 R2 Operations Manager Database Server"}.NodeName
            ResourceName = "[xSQLServer2012Install]MSSQLSERVER"
            RetryIntervalSec = 60
            RetryCount = 60
            Credential = $Node.InstallerServiceAccount
        }

        WaitForAll SC2012R2OMDW
        {
            NodeName = $AllNodes.Where{$_.Role -eq "System Center 2012 R2 Operations Manager Datawarehouse Server"}.NodeName
            ResourceName = "[xSQLServer2012Install]MSSQLSERVER"
            RetryIntervalSec = 60
            RetryCount = 60
            Credential = $Node.InstallerServiceAccount
        }



    }

    Node $AllNodes.Where{$_.Role -eq "System Center 2012 R2 Operations Manager Console"}.NodeName
    {
        WindowsFeature NET351
        {
            Name = "NET-Framework-Core"
            Ensure = "Present"
            Source = $Node.SourcePath +  "\WindowsServer2012R2\sources\sxs"
        }

        WindowsFeature NET45
        {
            Name = "NET-Framework-45-Core"
            Ensure = "Present"
        }

        xSQLServer2012Install SQLServer2012ManagementTools
        {
            DependsOn = "[WindowsFeature]NET351"
            SourcePath = $Node.SourcePath
            InstanceName = "NULL"
            Features = "SSMS,ADV_SSMS"
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
        }


    }
}

$SecurePassword = ConvertTo-SecureString -String "Pass@word1" -AsPlainText -Force
$InstallerServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!Installer", $SecurePassword)

$SystemCenter2012R2VirtualMachineManagerServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!vmm", $SecurePassword)

$ConfigurationData = @{
    AllNodes = @(
        @{
            NodeName = "*"
            PSDscAllowPlainTextPassword = $true

            InstallerServiceAccount = $InstallerServiceAccount

            SQLServer2012ProductKey = "748RB-X4T6B-MRM7V-RTVFF-CHC8H"
            SystemCenter2012ProductKey = "BXH69-M62YX-QQD6R-3GPWX-8WMFY"

            SourcePath = "\\RD01\Installer"

            RegisteredUser = "Microsoft Corporation"
            RegisteredOrganization = "Microsoft Corporation"
            
            SystemCenter2012R2VirtualMachineManagerServiceAccount = $SystemCenter2012R2VirtualMachineManagerServiceAccount
        }
        @{
            NodeName = "OMDB.contoso.com"
            Role = "System Center 2012 R2 Operations Manager Database Server"
        }
        @{
            NodeName = "OMDW.contoso.com"
            Role = "System Center 2012 R2 Operations Manager Datawarehouse Server"
        }
        @{
            NodeName = "OM01.contoso.com"
            Role = "System Center 2012 R2 Operations Manager Management Server"
        }
        @{
            NodeName = "RD01.contoso.com"
            Role = "System Center 2012 R2 Operations Manager Console"
        }
    )
}

SC2012R2OM -ConfigurationData $ConfigurationData
Start-DscConfiguration -Path .\SC2012R2OM -Verbose -Wait