﻿Configuration SC2012R2OM_HA_Static_AD
{
    Node $AllNodes.NodeName
    {
        LocalConfigurationManager {
            DebugMode = $true
        }
    }

    Import-DscResource -Module xActiveDirectory

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Active Directory Domain Controller"}}.NodeName
    {
        xADUser "SQL"
        {
            Ensure = "Present"
            DomainName = $Node.SQLServiceAccount.GetNetworkCredential().Domain
            UserName = $Node.SQLServiceAccount.GetNetworkCredential().UserName
            Password = $Node.SQLServiceAccount
            DomainAdministratorCredential = $Node.DomainAdministratorCredential
        }

        xADUser "OM"
        {
            Ensure = "Present"
            DomainName = $Node.SystemCenter2012R2OperationsManagerActionAccount.GetNetworkCredential().Domain
            UserName = $Node.SystemCenter2012R2OperationsManagerActionAccount.GetNetworkCredential().UserName
            Password = $Node.SystemCenter2012R2OperationsManagerActionAccount
            DomainAdministratorCredential = $Node.DomainAdministratorCredential
        }
    }
}

Configuration SC2012R2OM_HA_Static
{
    Node $AllNodes.NodeName
    {
        LocalConfigurationManager {
            DebugMode = $true
        }
    }

    Import-DscResource -Module xPDT
    Import-DscResource -Module xCredSSP

    Node $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("SQL Cluster Node")}}.NodeName
    {
        WindowsFeature "NET-Framework-Core"
        {
            Ensure = "Present"
            Name = "NET-Framework-Core"
            Source = $Node.SourcePath + "\WindowsServer2012R2.en\sources\sxs"
        }

        xSQLFCIPrepare "OMDB"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
            InstanceName = "OMDB"
            Features = "SQLENGINE"
            FailoverClusterNetworkName = "CLOMDB"
            SQLSvcAccount = $Node.SQLServiceAccount
            AgtSvcAccount = $Node.SQLServiceAccount
        }

        xSQLFCIPrepare "OMDW"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
            InstanceName = "OMDW"
            Features = "SQLENGINE"
            FailoverClusterNetworkName = "CLOMDW"
            SQLSvcAccount = $Node.SQLServiceAccount
            AgtSvcAccount = $Node.SQLServiceAccount
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "SQL Cluster Node 1"}}.NodeName
    {
        WaitForAll "OMDB"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("SQL Cluster Node")}}.NodeName
            ResourceName = "[xSQLFCIPrepare]OMDB"
            RetryIntervalSec = 60
            RetryCount = 120
            Credential = $Node.InstallerServiceAccount
        }

        WaitForAll "OMDW"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("SQL Cluster Node")}}.NodeName
            ResourceName = "[xSQLFCIPrepare]OMDW"
            RetryIntervalSec = 60
            RetryCount = 120
            Credential = $Node.InstallerServiceAccount
        }

        xSQLFCIComplete "OMDB"
        {
            DependsOn = "[WaitForAll]OMDB"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            InstanceName = "OMDB"
            Features = "SQLENGINE"
            FailoverClusterNetworkName = "CLOMDB"
            FailoverClusterIPAddress = "192.168.1.245"
            SQLSysAdminAccounts = @("CONTOSO\Administrator")
            InstallSQLDataDir = "I:"
            SQLUserDBLogDir = "J:\MSSQL11.OMDB\MSSQL\Data"
            SQLTempDBDir = "K:\MSSQL11.OMDB\MSSQL\Data"
            SQLTempDBLogDir = "J:\MSSQL11.OMDB\MSSQL\Data"
        }

        xSQLFCIComplete "OMDW"
        {
            DependsOn = "[WaitForAll]OMDW"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            InstanceName = "OMDW"
            Features = "SQLENGINE"
            FailoverClusterNetworkName = "CLOMDW"
            FailoverClusterIPAddress = "192.168.1.244"
            SQLSysAdminAccounts = @("CONTOSO\Administrator")
            InstallSQLDataDir = "L:"
            SQLUserDBLogDir = "M:\MSSQL11.OMDB\MSSQL\Data"
            SQLTempDBDir = "N:\MSSQL11.OMDB\MSSQL\Data"
            SQLTempDBLogDir = "M:\MSSQL11.OMDB\MSSQL\Data"
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("Management Server")}}.NodeName
    {
        Group "Admin"
        {
            Ensure = "Present"
            GroupName = (Get-WMIObject Win32_Group -filter "LocalAccount=True AND SID='S-1-5-32-544'" -ComputerName $Node.NodeName).Name
            MembersToInclude = $Node.SystemCenter2012R2OperationsManagerActionAccount.UserName
            Credential = $Node.InstallerServiceAccount
        }

        WindowsFeature "NET-Framework-45-Core"
        {
            Name = "NET-Framework-45-Core"
            Ensure = "Present"
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Management Server"}}.NodeName
    {
        xCredSSP Server
        {
            Ensure = "Present"
            Role = "Server"
        }

        xCredSSP Client
        {
            Ensure = "Present"
            Role = "Client"
            DelegateComputers = @($Node.NodeName)
        }
        
        WaitForAll "OMDB"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "SQL Cluster Node 1"}}.NodeName
            ResourceName = "[xSQLFCIComplete]OMDB"
            RetryIntervalSec = 60
            RetryCount = 120
            Credential = $Node.InstallerServiceAccount
        }

        WaitForAll "OMDW"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "SQL Cluster Node 1"}}.NodeName
            ResourceName = "[xSQLFCIComplete]OMDW"
            RetryIntervalSec = 60
            RetryCount = 120
            Credential = $Node.InstallerServiceAccount
        }

        xSC2012R2OMInstall "OMMS"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-45-Core",
                "[WaitForAll]OMDB",
                "[WaitForAll]OMDW"
            )
            Role = "Management Server"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            ManagementGroupName = $Node.SystemCenter2012R2OperationsManagerManagementGroupName
            ActionAccount = $Node.SystemCenter2012R2OperationsManagerActionAccount
            SqlServerInstance = "CLOMDB\OMDB"
            DwSqlServerInstance = "CLOMDW\OMDW"
            ProductKey = $Node.SystemCenter2012ProductKey
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Additional Management Server"}}.NodeName
    {
        WaitForAll "OMMS"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Management Server"}}.NodeName
            ResourceName = "[xSC2012R2OMInstall]OMMS"
            RetryIntervalSec = 60
            RetryCount = 120
            Credential = $Node.InstallerServiceAccount
        }

        xSC2012R2OMInstall "OMMS"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-45-Core",
                "[WaitForAll]OMMS"
            )
            Role = "Additional Management Server"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            ActionAccount = $Node.SystemCenter2012R2OperationsManagerActionAccount
            SqlServerInstance = "CLOMDB\OMDB"
            DwSqlServerInstance = "CLOMDW\OMDW"
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Reporting Server"}}.NodeName
    {
        WindowsFeature "NET-Framework-Core"
        {
            Ensure = "Present"
            Name = "NET-Framework-Core"
            Source = $Node.SourcePath + "\WindowsServer2012R2.en\sources\sxs"
        }

        WindowsFeature "NET-Framework-45-Core"
        {
            Name = "NET-Framework-45-Core"
            Ensure = "Present"
        }

        xSQLServerInstall "MSSQLSERVER"
        {
            DependsOn = "[WindowsFeature]NET-Framework-Core"
            SourcePath = $Node.SourcePath
            InstanceName = "MSSQLSERVER"
            Features = "SQLENGINE,RS,AS"
            Credential = $Node.InstallerServiceAccount
        }

        WaitForAll "OMMS"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Management Server"}}.NodeName
            ResourceName = "[xSC2012R2OMInstall]OMMS"
            RetryIntervalSec = 60
            RetryCount = 120
            Credential = $Node.InstallerServiceAccount
        }

        xSC2012R2OMInstall "OMRS"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core",
                "[WindowsFeature]NET-Framework-45-Core",
                "[xSQLServerInstall]MSSQLSERVER",
                "[WaitForAll]OMMS"
            )
            Role = "Reporting Server"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            ActionAccount = $Node.SystemCenter2012R2OperationsManagerActionAccount
            ManagementServer = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Management Server"}}.NodeName
            SRSInstance = $Node.NodeName
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Web Console Server"}}.NodeName
    {
        WindowsFeature "NET-Framework-Core"
        {
            Ensure = "Present"
            Name = "NET-Framework-Core"
            Source = $Node.SourcePath + "\WindowsServer2012R2.en\sources\sxs"
        }

        WindowsFeature "NET-Framework-45-Core"
        {
            Name = "NET-Framework-45-Core"
            Ensure = "Present"
        }

        WindowsFeature "Web-WebServer"
        {
            Ensure = "Present"
            Name = "Web-WebServer"
        }

        WindowsFeature "Web-Default-Doc"
        {
            Ensure = "Present"
            Name = "Web-Default-Doc"
        }

        WindowsFeature "Web-Dir-Browsing"
        {
            Ensure = "Present"
            Name = "Web-Dir-Browsing"
        }

        WindowsFeature "Web-Http-Errors"
        {
            Ensure = "Present"
            Name = "Web-Http-Errors"
        }

        WindowsFeature "Web-Static-Content"
        {
            Ensure = "Present"
            Name = "Web-Static-Content"
        }

        WindowsFeature "Web-Http-Logging"
        {
            Ensure = "Present"
            Name = "Web-Http-Logging"
        }

        WindowsFeature "Web-Stat-Compression"
        {
            Ensure = "Present"
            Name = "Web-Stat-Compression"
        }

        WindowsFeature "Web-Filtering"
        {
            Ensure = "Present"
            Name = "Web-Filtering"
        }

        WindowsFeature "Web-Request-Monitor"
        {
            Ensure = "Present"
            Name = "Web-Request-Monitor"
        }

        WindowsFeature "Web-Windows-Auth"
        {
            Ensure = "Present"
            Name = "Web-Windows-Auth"
        }

        WindowsFeature "Web-Asp-Net"
        {
            Ensure = "Present"
            Name = "Web-Asp-Net"
        }

        WindowsFeature "Web-Net-Ext"
        {
            Ensure = "Present"
            Name = "Web-Net-Ext"
        }

        WindowsFeature "Web-Asp-Net45"
        {
            Ensure = "Present"
            Name = "Web-Asp-Net45"
        }

        WindowsFeature "Web-Net-Ext45"
        {
            Ensure = "Present"
            Name = "Web-Net-Ext45"
        }

        WindowsFeature "Web-ISAPI-Ext"
        {
            Ensure = "Present"
            Name = "Web-ISAPI-Ext"
        }

        WindowsFeature "Web-ISAPI-Filter"
        {
            Ensure = "Present"
            Name = "Web-ISAPI-Filter"
        }

        WindowsFeature "Web-Mgmt-Console"
        {
            Ensure = "Present"
            Name = "Web-Mgmt-Console"
        }

        WindowsFeature "Web-Metabase"
        {
            Ensure = "Present"
            Name = "Web-Metabase"
        }

        WindowsFeature "NET-Framework-45-ASPNET"
        {
            Ensure = "Present"
            Name = "NET-Framework-45-ASPNET"
        }

        WindowsFeature "NET-WCF-HTTP-Activation45"
        {
            Ensure = "Present"
            Name = "NET-WCF-HTTP-Activation45"
        }

        WindowsFeature "WAS-Process-Model"
        {
            Ensure = "Present"
            Name = "WAS-Process-Model"
        }

        WindowsFeature "WAS-Config-APIs"
        {
            Ensure = "Present"
            Name = "WAS-Config-APIs"
        }

        xPDTPrerequisiteInstall "SQL2012CLR"
        {
            Name = "SQL Server 2012 System CLR Types"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "RV2012"
        {
            Name = "Report Viewer 2012 Redistributable"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        WaitForAll "OMMS"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Management Server"}}.NodeName
            ResourceName = "[xSC2012R2OMInstall]OMMS"
            RetryIntervalSec = 60
            RetryCount = 120
            Credential = $Node.InstallerServiceAccount
        }

        xSC2012R2OMInstall "OMWC"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core",
                "[WindowsFeature]NET-Framework-45-Core",
                "[WindowsFeature]Web-WebServer",
                "[WindowsFeature]Web-Default-Doc",
                "[WindowsFeature]Web-Dir-Browsing",
                "[WindowsFeature]Web-Http-Errors",
                "[WindowsFeature]Web-Static-Content",
                "[WindowsFeature]Web-Http-Logging",
                "[WindowsFeature]Web-Stat-Compression",
                "[WindowsFeature]Web-Filtering",
                "[WindowsFeature]Web-Request-Monitor",
                "[WindowsFeature]Web-Windows-Auth",
                "[WindowsFeature]Web-Asp-Net",
                "[WindowsFeature]Web-Net-Ext",
                "[WindowsFeature]Web-Asp-Net45",
                "[WindowsFeature]Web-Net-Ext45",
                "[WindowsFeature]Web-ISAPI-Ext",
                "[WindowsFeature]Web-ISAPI-Filter",
                "[WindowsFeature]Web-Mgmt-Console",
                "[WindowsFeature]Web-Metabase",
                "[WindowsFeature]NET-Framework-45-ASPNET",
                "[WindowsFeature]NET-WCF-HTTP-Activation45",
                "[WindowsFeature]WAS-Process-Model",
                "[WindowsFeature]WAS-Config-APIs",
                "[xPDTPrerequisiteInstall]SQL2012CLR",
                "[xPDTPrerequisiteInstall]RV2012",
                "[WaitForAll]OMMS"
            )
            Role = "Web Console Server"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            ManagementServer = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Management Server"}}.NodeName
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("Console Node")}}.NodeName
    {
        WindowsFeature "NET-Framework-Core"
        {
            Name = "NET-Framework-Core"
            Ensure = "Present"
            Source = $Node.SourcePath +  "\WindowsServer2012R2\sources\sxs"
        }

        WindowsFeature "NET-Framework-45-Core"
        {
            Name = "NET-Framework-45-Core"
            Ensure = "Present"
        }

        xSQLServerInstall "SQLMT"
        {
            DependsOn = "[WindowsFeature]NET-Framework-Core"
            SourcePath = $Node.SourcePath
            InstanceName = "NULL"
            Features = "SSMS,ADV_SSMS"
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
        }

        xPDTPrerequisiteInstall "SQL2012CLR"
        {
            Name = "SQL Server 2012 System CLR Types"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "RV2012"
        {
            Name = "Report Viewer 2012 Redistributable"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xSC2012R2OMInstall "OMC"
        {
            DependsOn = @(
                "[xPDTPrerequisiteInstall]SQL2012CLR",
                "[xPDTPrerequisiteInstall]RV2012"
            )
            Role = "Console"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }
    }
}

$SecurePassword = ConvertTo-SecureString -String "Pass@word1" -AsPlainText -Force
$DomainAdministratorCredential = New-Object System.Management.Automation.PSCredential ("CONTOSO\Administrator", $SecurePassword)
$InstallerServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!Installer", $SecurePassword)
$SQLServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!sql", $SecurePassword)
$SystemCenter2012R2OperationsManagerActionAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!om", $SecurePassword)

$ConfigurationData = @{
    AllNodes = @(
        @{
            NodeName = "*"
            PSDscAllowPlainTextPassword = $true

            DomainAdministratorCredential = $DomainAdministratorCredential
            InstallerServiceAccount = $InstallerServiceAccount

            SQLServer2012ProductKey = "748RB-X4T6B-MRM7V-RTVFF-CHC8H"
            SystemCenter2012ProductKey = "BXH69-M62YX-QQD6R-3GPWX-8WMFY"

            SourcePath = "\\RD01\Installer"

            RegisteredUser = "Microsoft Corporation"
            RegisteredOrganization = "Microsoft Corporation"
            
            SQLServiceAccount = $SQLServiceAccount

            SystemCenter2012R2OperationsManagerActionAccount = $SystemCenter2012R2OperationsManagerActionAccount
            SystemCenter2012R2OperationsManagerManagementGroupName = "OM_Contoso"
        }
        @{
            NodeName = "DC01.contoso.com"
            Roles = @("Active Directory Domain Controller")
        }
        @{
            NodeName = "CLDBA.contoso.com"
            Roles = @("SQL Cluster Node 1")
        }
        @{
            NodeName = "CLDBB.contoso.com"
            Roles = @("SQL Cluster Node 2")
        }
        @{
            NodeName = "CLDBC.contoso.com"
            Roles = @("SQL Cluster Node 3")
        }
        @{
            NodeName = "CLDBD.contoso.com"
            Roles = @("SQL Cluster Node 4")
        }
        @{
            NodeName = "OM01.contoso.com"
            Roles = @("Management Server")
        }
        @{
            NodeName = "OM02.contoso.com"
            Roles = @("Additional Management Server")
        }
        @{
            NodeName = "OM03.contoso.com"
            Roles = @("Additional Management Server")
        }
        @{
            NodeName = "OM04.contoso.com"
            Roles = @("Web Console Server")
        }
        @{
            NodeName = "OMRS.contoso.com"
            Roles = @("Reporting Server")
        }
        @{
            NodeName = "RD01.contoso.com"
            Roles = @("Console Node 1")
        }
    )
}

foreach($Node in $ConfigurationData.AllNodes)
{
    if($Node.NodeName -ne "*")
    {
        Start-Process -FilePath "robocopy.exe" -ArgumentList ("`"C:\Program Files\WindowsPowerShell\Modules`" `"\\" + $Node.NodeName + "\c$\Program Files\WindowsPowerShell\Modules`" /e /purge /xf") -NoNewWindow -Wait
    }
}

SC2012R2OM_HA_Static_AD -ConfigurationData $ConfigurationData
SC2012R2OM_HA_Static -ConfigurationData $ConfigurationData
Set-DscLocalConfigurationManager -Path .\SC2012R2OM_HA_Static_AD -Verbose
Set-DscLocalConfigurationManager -Path .\SC2012R2OM_HA_Static -Verbose
Start-DscConfiguration -Path .\SC2012R2OM_HA_Static_AD -Verbose -Wait
Start-DscConfiguration -Path .\SC2012R2OM_HA_Static -Verbose -Wait