﻿#requires -version 5

Configuration SC2012R2VMM
{
    Import-DscResource -Module xPDT
    
    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "System Center 2012 R2 Virtual Machine Manager Database Server"}}.NodeName
    {
        LocalConfigurationManager
        {
            DebugMode = $true
        }

        WindowsFeature "NET-Framework-Core"
        {
            Name = "NET-Framework-Core"
            Ensure = "Present"
            Source = $Node.SourcePath +  "\WindowsServer2012R2\sources\sxs"
        }

        xSQLServerInstall "MSSQLSERVER"
        {
            DependsOn = "[WindowsFeature]NET-Framework-Core"
            SourcePath = $Node.SourcePath
            InstanceName = "MSSQLSERVER"
            Features = "SQLENGINE"
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
            SQLSysAdminAccounts = @("CONTOSO\Administrator")
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "System Center 2012 R2 Virtual Machine Manager Management Server"}}.NodeName
    {
        LocalConfigurationManager
        {
            DebugMode = $true
        }

        Group "Admin"
        {
            Ensure = "Present"
            GroupName = "Administrators"
            MembersToInclude = $Node.SystemCenter2012R2VirtualMachineManagerServiceAccount.UserName
            Credential = $Node.InstallerServiceAccount
        }

        WindowsFeature "NET-Framework-45-Core"
        {
            Name = "NET-Framework-45-Core"
            Ensure = "Present"
        }

        xPDTPrerequisiteInstall "SQLServer2012NativeClient"
        {
            Name = "SQL Server 2012 Native Client"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "SQLServer2012CommandLineUtilities"
        {
            DependsOn = "[xPDTPrerequisiteInstall]SQLServer2012NativeClient"
            Name = "SQL Server 2012 Command Line Utilities"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "WindowsDeploymentTools81"
        {
            Name = "Windows Deployment Tools 8.1"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "WindowsPreinstallationEnvironment81"
        {
            Name = "Windows Preinstallation Environment 8.1"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        WaitForAll "VMMDB"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "System Center 2012 R2 Virtual Machine Manager Database Server"}}.NodeName
            ResourceName = "[xSQLServerInstall]MSSQLSERVER"
            RetryIntervalSec = 60
            RetryCount = 60
            Credential = $Node.InstallerServiceAccount
        }

        xSCVMMManagementServer "SC2012R2VMMManagementServer"
        {
            Ensure = "Present"
            DependsOn = @(
                "[WindowsFeature]NET-Framework-45-Core",
                "[xPDTPrerequisiteInstall]SQLServer2012NativeClient",
                "[xPDTPrerequisiteInstall]SQLServer2012CommandLineUtilities",
                "[xPDTPrerequisiteInstall]WindowsDeploymentTools81",
                "[xPDTPrerequisiteInstall]WindowsPreinstallationEnvironment81",
                "[WaitForAll]VMMDB"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            SqlMachineName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "System Center 2012 R2 Virtual Machine Manager Database Server"}}.NodeName
            SqlInstanceName = "MSSQLSERVER"
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "System Center 2012 R2 Virtual Machine Manager Console"}}.NodeName
    {
        LocalConfigurationManager
        {
            DebugMode = $true
        }

        WindowsFeature "NET-Framework-Core"
        {
            Name = "NET-Framework-Core"
            Ensure = "Present"
            Source = $Node.SourcePath +  "\WindowsServer2012R2\sources\sxs"
        }

        WindowsFeature "NET-Framework-45-Core"
        {
            Name = "NET-Framework-45-Core"
            Ensure = "Present"
        }

        xSQLServerInstall "SQLServer2012ManagementTools"
        {
            DependsOn = "[WindowsFeature]NET-Framework-Core"
            SourcePath = $Node.SourcePath
            InstanceName = "NULL"
            Features = "SSMS,ADV_SSMS"
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
        }

        xSCVMMConsole "SC2012R2VMMConsole"
        {
            Ensure = "Present"
            DependsOn = "[WindowsFeature]NET-Framework-45-Core"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }
    }
}

$SecurePassword = ConvertTo-SecureString -String "Pass@word1" -AsPlainText -Force
$InstallerServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!Installer", $SecurePassword)

$ConfigurationData = @{
    AllNodes = @(
        @{
            NodeName = "*"
            PSDscAllowPlainTextPassword = $true

            InstallerServiceAccount = $InstallerServiceAccount

            SQLServer2012ProductKey = "748RB-X4T6B-MRM7V-RTVFF-CHC8H"
            SystemCenter2012ProductKey = "BXH69-M62YX-QQD6R-3GPWX-8WMFY"

            SourcePath = "\\RD01\Installer"

            RegisteredUser = "Microsoft Corporation"
            RegisteredOrganization = "Microsoft Corporation"
        }
        @{
            NodeName = "VMMDB.contoso.com"
            Roles = @("System Center 2012 R2 Virtual Machine Manager Database Server")
        }
        @{
            NodeName = "VMM01.contoso.com"
            Roles = @("System Center 2012 R2 Virtual Machine Manager Management Server")
        }
        @{
            NodeName = "RD01.contoso.com"
            Roles = @("System Center 2012 R2 Virtual Machine Manager Console")
        }
    )
}

foreach($Node in $ConfigurationData.AllNodes)
{
    if($Node.NodeName -ne "*")
    {
        Start-Process -FilePath "robocopy.exe" -ArgumentList ("`"C:\Program Files\WindowsPowerShell\Modules`" `"\\" + $Node.NodeName + "\c$\Program Files\WindowsPowerShell\Modules`" /e /purge /xf") -NoNewWindow -Wait
    }
}

SC2012R2VMM -ConfigurationData $ConfigurationData
Set-DscLocalConfigurationManager -Path .\SC2012R2VMM -Verbose
Start-DscConfiguration -Path .\SC2012R2VMM -Wait -Verbose