﻿Configuration SC2012R2VMM_HA_Static_AD
{
    Node $AllNodes.NodeName
    {
        LocalConfigurationManager {
            DebugMode = $true
        }
    }

    Import-DscResource -Module xActiveDirectory

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Active Directory Domain Controller"}}.NodeName
    {
        xADUser "SQL"
        {
            Ensure = "Present"
            DomainName = $Node.SQLServiceAccount.GetNetworkCredential().Domain
            UserName = $Node.SQLServiceAccount.GetNetworkCredential().UserName
            Password = $Node.SQLServiceAccount
            DomainAdministratorCredential = $Node.DomainAdministratorCredential
        }

        xADUser "VMM"
        {
            Ensure = "Present"
            DomainName = $Node.SystemCenter2012R2VirtualMachineManagerServiceAccount.GetNetworkCredential().Domain
            UserName = $Node.SystemCenter2012R2VirtualMachineManagerServiceAccount.GetNetworkCredential().UserName
            Password = $Node.SystemCenter2012R2VirtualMachineManagerServiceAccount
            DomainAdministratorCredential = $Node.DomainAdministratorCredential
        }

        Script "VMMDKM"
        {
#            DependsOn = "[xADUser]VMM"
            GetScript = {
                if((Get-ADObject -Filter {(ObjectClass -eq "container") -and (Name -eq "VMMDKM")}).DistinguishedName -eq "CN=VMMDKM,DC=contoso,DC=com")
                {
                    $acl = Get-Acl -Path "AD:CN=VMMDKM,DC=contoso,DC=com"
                    if($acl.Access | Where-Object {($_.ActiveDirectoryRights -eq "GenericAll") -and ($_.ObjectType -eq "00000000-0000-0000-0000-000000000000") -and ($_.AccessControlType -eq "Allow") -and ($_.IdentityReference -eq "CONTOSO\!vmm")})
                    {
                        return @{Result = $true}
                    }
                    else
                    {
                        return @{Result = $false}
                    }
                }
                else
                {
                    return @{Result = $false}
                }
            }
            SetScript = {
                if(!((Get-ADObject -Filter {(ObjectClass -eq "container") -and (Name -eq "VMMDKM")}).DistinguishedName -eq "CN=VMMDKM,DC=contoso,DC=com"))
                {
                    New-ADObject -Name "VMMDKM" -Type Container -Path "DC=contoso,DC=com"
                }
                if((Get-ADObject -Filter {(ObjectClass -eq "container") -and (Name -eq "VMMDKM")}).DistinguishedName -eq "CN=VMMDKM,DC=contoso,DC=com")
                {
                    $acl = Get-Acl -Path "AD:CN=VMMDKM,DC=contoso,DC=com"
                    if(!($acl.Access | Where-Object {($_.ActiveDirectoryRights -eq "GenericAll") -and ($_.ObjectType -eq "00000000-0000-0000-0000-000000000000") -and ($_.AccessControlType -eq "Allow") -and ($_.IdentityReference -eq "CONTOSO\!vmm")}))
                    {
                        $VMMSA = (Get-ADUser -Identity "!vmm").SID
                        $nullGuid = New-Object Guid 00000000-0000-0000-0000-000000000000
                        $ace = New-Object System.DirectoryServices.ActiveDirectoryAccessRule $VMMSA,"GenericAll","Allow","All",$nullGuid
                        $acl.AddAccessRule($ace)
                        Set-Acl -Path "AD:CN=VMMDKM,DC=contoso,DC=com" -AclObject $acl
                    }
                }
            }
            TestScript = {
                if((Get-ADObject -Filter {(ObjectClass -eq "container") -and (Name -eq "VMMDKM")}).DistinguishedName -eq "CN=VMMDKM,DC=contoso,DC=com")
                {
                    $acl = Get-Acl -Path "AD:CN=VMMDKM,DC=contoso,DC=com"
                    if($acl.Access | Where-Object {($_.ActiveDirectoryRights -eq "GenericAll") -and ($_.ObjectType -eq "00000000-0000-0000-0000-000000000000") -and ($_.AccessControlType -eq "Allow") -and ($_.IdentityReference -eq "CONTOSO\!vmm")})
                    {
                        return $true
                    }
                    else
                    {
                        return $false
                    }
                }
                else
                {
                    return $false
                }
            }
            Credential = $Node.DomainAdminCredential
        }
    }
}

Configuration SC2012R2VMM_HA_Static
{
    Node $AllNodes.NodeName
    {
        LocalConfigurationManager {
            DebugMode = $true
        }
    }

    Import-DscResource -Module xPDT

    Node $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("SQL Cluster Node")}}.NodeName
    {
        WindowsFeature NET351
        {
            Ensure = "Present"
            Name = "NET-Framework-Core"
            Source = $Node.SourcePath + "\WindowsServer2012R2.en\sources\sxs"
        }

#        WaitForAll "ADSQL"
#        {
#            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Active Directory Domain Controller"}}.NodeName
#            ResourceName = "[xADUser]SQL"
#            RetryIntervalSec = 60
#            RetryCount = 60
#            Credential = $Node.DomainAdministratorCredential
#        }

        xSQLFCIPrepare "VMMDB"
        {
#            DependsOn = @(
#                "[WindowsFeature]NET351",
#                "[WaitForAll]ADSQL"
#            )
            DependsOn = @(
                "[WindowsFeature]NET351"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
            InstanceName = "VMMDB"
            Features = "SQLENGINE"
            FailoverClusterNetworkName = "CLVMMDB"
            SQLSvcAccount = $Node.SQLServiceAccount
            AgtSvcAccount = $Node.SQLServiceAccount
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "SQL Cluster Node 1"}}.NodeName
    {
        WaitForAll "VMMDB"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("SQL Cluster Node")}}.NodeName
            ResourceName = "[xSQLFCIPrepare]VMMDB"
            RetryIntervalSec = 60
            RetryCount = 60
            Credential = $Node.InstallerServiceAccount
        }

        xSQLFCIComplete "VMMDB"
        {
            DependsOn = "[WaitForAll]VMMDB"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            InstanceName = "VMMDB"
            Features = "SQLENGINE"
            FailoverClusterNetworkName = "CLVMMDB"
            FailoverClusterIPAddress = "192.168.1.247"
            SQLSysAdminAccounts = @("CONTOSO\Administrator")
            InstallSQLDataDir = "F:"
            SQLUserDBLogDir = "G:\MSSQL11.VMMDB\MSSQL\Data"
            SQLTempDBDir = "H:\MSSQL11.VMMDB\MSSQL\Data"
            SQLTempDBLogDir = "G:\MSSQL11.VMMDB\MSSQL\Data"
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("VMM Cluster Node")}}.NodeName
    {

#        WaitForAll "ADVMM"
#        {
#            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Active Directory Domain Controller"}}.NodeName
#            ResourceName = "[xADUser]VMM"
#            RetryIntervalSec = 60
#            RetryCount = 60
#            Credential = $Node.DomainAdministratorCredential
#        }
        
        Group "Admin"
        {
#            DependsOn = "[WaitForAll]ADVMM"
            Ensure = "Present"
            GroupName = (Get-WMIObject Win32_Group -filter "LocalAccount=True AND SID='S-1-5-32-544'" -ComputerName $Node.NodeName).Name
            MembersToInclude = $Node.SystemCenter2012R2VirtualMachineManagerServiceAccount.UserName
            Credential = $Node.InstallerServiceAccount
        }

        WindowsFeature "NET45"
        {
            Name = "NET-Framework-45-Core"
            Ensure = "Present"
        }

        xPDTPrerequisiteInstall "SQLServer2012NativeClient"
        {
            Name = "SQL Server 2012 Native Client"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "SQLServer2012CommandLineUtilities"
        {
            DependsOn = "[xPDTPrerequisiteInstall]SQLServer2012NativeClient"
            Name = "SQL Server 2012 Command Line Utilities"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "WindowsDeploymentTools81"
        {
            Name = "Windows Deployment Tools 8.1"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "WindowsPreinstallationEnvironment81"
        {
            Name = "Windows Preinstallation Environment 8.1"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "VMM Cluster Node 1"}}.NodeName
    {
#        WaitForAll "ADVMMDKM"
#        {
#            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Active Directory Domain Controller"}}.NodeName
#            ResourceName = "[Script]VMMDKM"
#            RetryIntervalSec = 60
#            RetryCount = 60
#            Credential = $Node.DomainAdministratorCredential
#        }
        
        WaitForAll "VMMDB"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "SQL Cluster Node 1"}}.NodeName
            ResourceName = "[xSQLFCIComplete]VMMDB"
            RetryIntervalSec = 60
            RetryCount = 60
            Credential = $Node.InstallerServiceAccount
        }

        xSC2012R2VMMInstall "VMMMS"
        {
            DependsOn = @(
                "[WindowsFeature]NET45",
                "[xPDTPrerequisiteInstall]SQLServer2012NativeClient",
                "[xPDTPrerequisiteInstall]SQLServer2012CommandLineUtilities",
                "[xPDTPrerequisiteInstall]WindowsDeploymentTools81",
                "[xPDTPrerequisiteInstall]WindowsPreinstallationEnvironment81",
                "[WaitForAll]VMMDB"
            )
#            DependsOn = @(
#                "[WindowsFeature]NET45",
#                "[xPDTPrerequisiteInstall]SQLServer2012NativeClient",
#                "[xPDTPrerequisiteInstall]SQLServer2012CommandLineUtilities",
#                "[xPDTPrerequisiteInstall]WindowsDeploymentTools81",
#                "[xPDTPrerequisiteInstall]WindowsPreinstallationEnvironment81",
#                "[WaitForAll]ADVMMDKM",
#                "[WaitForAll]VMMDB"
#            )
            Role = "Cluster Active Management Server"
            SourcePath = $Node.SourcePath
            vmmService = $Node.SystemCenter2012R2VirtualMachineManagerServiceAccount
            Credential = $Node.InstallerServiceAccount
            SqlInstanceName = "VMMDB"
            SqlMachineName = "CLVMMDB"
            TopContainerName = "CN=VMMDKM,DC=contoso,DC=com"
            VmmServerName = "CLVMM"
            VmmStaticIPAddress = "192.168.1.246"
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "VMM Cluster Node 2"}}.NodeName
    {
        WaitForAll "VMMMS"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "VMM Cluster Node 1"}}.NodeName
            ResourceName = "[xSC2012R2VMMInstall]VMMMS"
            RetryIntervalSec = 60
            RetryCount = 60
            Credential = $Node.InstallerServiceAccount
        }

        xSC2012R2VMMInstall "VMMMS"
        {
            DependsOn = @(
                "[WindowsFeature]NET45",
                "[xPDTPrerequisiteInstall]SQLServer2012NativeClient",
                "[xPDTPrerequisiteInstall]SQLServer2012CommandLineUtilities",
                "[xPDTPrerequisiteInstall]WindowsDeploymentTools81",
                "[xPDTPrerequisiteInstall]WindowsPreinstallationEnvironment81",
                "[WaitForAll]VMMMS"
            )
            Role = "Cluster Passive Management Server"
            SourcePath = $Node.SourcePath
            vmmService = $Node.SystemCenter2012R2VirtualMachineManagerServiceAccount
            Credential = $Node.InstallerServiceAccount
            SqlInstanceName = "VMMDB"
            SqlMachineName = "CLVMMDB"
            TopContainerName = "CN=VMMDKM,DC=contoso,DC=com"
            VmmServerName = "CLVMM"
            VmmStaticIPAddress = "192.168.1.246"
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("Console Node")}}.NodeName
    {
        WindowsFeature "NET351"
        {
            Name = "NET-Framework-Core"
            Ensure = "Present"
            Source = $Node.SourcePath +  "\WindowsServer2012R2\sources\sxs"
        }

        WindowsFeature "NET45"
        {
            Name = "NET-Framework-45-Core"
            Ensure = "Present"
        }

        xSQLServerInstall "SQLMT"
        {
            DependsOn = "[WindowsFeature]NET351"
            SourcePath = $Node.SourcePath
            InstanceName = "NULL"
            Features = "SSMS,ADV_SSMS"
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
        }

        xSC2012R2VMMInstall "VMMC"
        {
            DependsOn = "[WindowsFeature]NET45"
            Role = "Console"
            SourcePath = "\\RD01\Installer"
            Credential = $Node.InstallerServiceAccount
        }
    }
}

# Give !sql SPN permissions

$SecurePassword = ConvertTo-SecureString -String "Pass@word1" -AsPlainText -Force
$DomainAdministratorCredential = New-Object System.Management.Automation.PSCredential ("CONTOSO\Administrator", $SecurePassword)
$InstallerServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!Installer", $SecurePassword)
$SQLServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!sql", $SecurePassword)
$SystemCenter2012R2VirtualMachineManagerServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!vmm", $SecurePassword)

$ConfigurationData = @{
    AllNodes = @(
        @{
            NodeName = "*"
            PSDscAllowPlainTextPassword = $true

            DomainAdministratorCredential = $DomainAdministratorCredential
            InstallerServiceAccount = $InstallerServiceAccount

            SQLServer2012ProductKey = "748RB-X4T6B-MRM7V-RTVFF-CHC8H"
            SystemCenter2012ProductKey = "BXH69-M62YX-QQD6R-3GPWX-8WMFY"

            SourcePath = "\\RD01\Installer"

            RegisteredUser = "Microsoft Corporation"
            RegisteredOrganization = "Microsoft Corporation"
            
            SQLServiceAccount = $SQLServiceAccount
            SystemCenter2012R2VirtualMachineManagerServiceAccount = $SystemCenter2012R2VirtualMachineManagerServiceAccount
        }
        @{
            NodeName = "DC01.contoso.com"
            Roles = @("Active Directory Domain Controller")
        }
        @{
            NodeName = "CLDBA.contoso.com"
            Roles = @("SQL Cluster Node 1")
        }
        @{
            NodeName = "CLDBB.contoso.com"
            Roles = @("SQL Cluster Node 2")
        }
        @{
            NodeName = "CLDBC.contoso.com"
            Roles = @("SQL Cluster Node 3")
        }
        @{
            NodeName = "CLDBD.contoso.com"
            Roles = @("SQL Cluster Node 4")
        }
        @{
            NodeName = "CLVMMA.contoso.com"
            Roles = @("VMM Cluster Node 1")
        }
        @{
            NodeName = "CLVMMB.contoso.com"
            Roles = @("VMM Cluster Node 2")
        }
        @{
            NodeName = "RD01.contoso.com"
            Roles = @("Console Node 1")
        }
    )
}

foreach($Node in $ConfigurationData.AllNodes)
{
    if($Node.NodeName -ne "*")
    {
        Start-Process -FilePath "robocopy.exe" -ArgumentList ("`"C:\Program Files\WindowsPowerShell\Modules`" `"\\" + $Node.NodeName + "\c$\Program Files\WindowsPowerShell\Modules`" /e /purge /xf") -NoNewWindow -Wait
    }
}

SC2012R2VMM_HA_Static_AD -ConfigurationData $ConfigurationData
SC2012R2VMM_HA_Static -ConfigurationData $ConfigurationData
Set-DscLocalConfigurationManager -Path .\SC2012R2VMM_HA_Static_AD -Verbose
Set-DscLocalConfigurationManager -Path .\SC2012R2VMM_HA_Static -Verbose
Start-DscConfiguration -Path .\SC2012R2VMM_HA_Static_AD -Verbose -Wait
Start-DscConfiguration -Path .\SC2012R2VMM_HA_Static -Verbose -Wait