﻿Configuration SQLFCI
{
    Import-DscResource -Module xPDT    

    Node $AllNodes.NodeName
    {
        LocalConfigurationManager
        {
            DebugMode = $true
        }

        if($Node.SQLFCIs)
        {
            WindowsFeature NET351
            {
                Ensure = "Present"
                Name = "NET-Framework-Core"
                Source = $Node.SourcePath + "\WindowsServer2012R2.en\sources\sxs"
            }
        }

        foreach($SQLFCI in $Node.SQLFCIs)
        {
            xSQLFCIPrepare $SQLFCI.InstanceName
            {
                DependsOn = "[WindowsFeature]NET351"
                SourcePath = $Node.SourcePath
                SourceFolder = $SQLFCI.SourceFolder
                Credential = $Node.InstallerServiceAccount
                PID = $SQLFCI.PID
                UpdateEnabled = $SQLFCI.UpdateEnabled
                UpdateSource = $SQLFCI.UpdateSource
                SQMReporting = $SQLFCI.SQMReporting
                ErrorReporting = $SQLFCI.ErrorReporting
                InstanceName = $SQLFCI.InstanceName
                InstanceID = $SQLFCI.InstanceID
                Features = $SQLFCI.Features
                FailoverClusterNetworkName = $SQLFCI.FailoverClusterNetworkName
                InstallSharedDir = $SQLFCI.InstallSharedDir
                InstallSharedWOWDir = $SQLFCI.InstallSharedWOWDir
                InstanceDir = $SQLFCI.InstanceDir
                SQLSvcAccount = $SQLFCI.SQLSvcAccount
                AgtSvcAccount = $SQLFCI.AgtSvcAccount
                ASSvcAccount = $SQLFCI.ASSvcAccount
                ISSvcAccount = $SQLFCI.ISSvcAccount
                ISFileSystemFolder = $SQLFCI.ISFileSystemFolder
            }
        
            $FCINodes = @()
            foreach($xNode in $AllNodes)
            {
                foreach($xSQLFCI in $xNode.SQLFCIs)
                {
                    if($xSQLFCI.InstanceName -eq $SQLFCI.InstanceName)
                    {
                        $FCINodes += $xNode.NodeName
                    }
                }
            }

            $PreferredNodes = @()
            foreach($xNode in $AllNodes)
            {
                foreach($xSQLFCI in $xNode.SQLFCIs)
                {
                    if(($xSQLFCI.InstanceName -eq $SQLFCI.InstanceName) -and ($xSQLFCI.Preferred))
                    {
                        $PreferredNodes += $xNode.NodeName
                    }
                }
            }            

            if(!$PreferredNodes)
            {
                $CompleteNode = ($FCINodes | Sort-Object)[0]
            }
            else
            {
                $CompleteNode = $PreferredNodes[0]
            }

            if($Node.NodeName -eq $CompleteNode)
            {
                WaitForAll $SQLFCI.InstanceName
                {
                    NodeName = $FCINodes
                    ResourceName = "[xSQLFCIPrepare]" + $SQLFCI.InstanceName
                    RetryIntervalSec = 60
                    RetryCount = 60
                    Credential = $Node.InstallerServiceAccount
                }

                xSQLFCIComplete $SQLFCI.InstanceName
                {
                    DependsOn = @(("[WaitForAll]" + $SQLFCI.InstanceName))
                    SourcePath = $Node.SourcePath
                    SourceFolder = $SQLFCI.SourceFolder
                    Credential = $Node.InstallerServiceAccount
                    InstanceName = $SQLFCI.InstanceName
                    Features = $SQLFCI.Features
                    FailoverClusterGroup = $SQLFCI.FailoverClusterGroup
                    FailoverClusterNetworkName = $SQLFCI.FailoverClusterNetworkName
                    FailoverClusterIPAddress = $SQLFCI.FailoverClusterIPAddress
                    SQLPort = $SQLFCI.SQLPort
                    SQLCollation = $SQLFCI.SQLCollation
                    SQLSysAdminAccounts = $SQLFCI.SQLSysAdminAccounts
                    SecurityMode = $SQLFCI.SecurityMode
                    SAPwd = $SQLFCI.SAPwd
                    InstallSQLDataDir = $SQLFCI.InstallSQLDataDir
                    SQLUserDBDir = $SQLFCI.SQLUserDBDir
                    SQLUserDBLogDir = $SQLFCI.SQLUserDBLogDir
                    SQLTempDBDir = $SQLFCI.SQLTempDBDir
                    SQLTempDBLogDir = $SQLFCI.SQLTempDBLogDir
                    SQLBackupDir = $SQLFCI.SQLBackupDir
                    SQLMaxMemory = $SQLFCI.SQLMaxMemory
                    ASCollation = $SQLFCI.ASCollation
                    ASSysAdminAccounts = $SQLFCI.ASSysAdminAccounts
                    ASDataDir = $SQLFCI.ASDataDir
                    ASLogDir = $SQLFCI.ASLogDir
                    ASBackupDir = $SQLFCI.ASBackupDir
                    ASTempDir = $SQLFCI.ASTempDir
                    ASConfigDir = $SQLFCI.ASConfigDir
                    ISFileSystemFolder = $SQLFCI.ISFileSystemFolder
                }
            }
        }
    }
}

$SecurePassword = ConvertTo-SecureString -String "Pass@word1" -AsPlainText -Force
$InstallerServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!Installer", $SecurePassword)
$SQLServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!sql", $SecurePassword)

$ConfigurationData = @{
    AllNodes = @(
        @{
            NodeName = "*"
            SourcePath = "\\RD01\Installer"
            InstallerServiceAccount = $InstallerServiceAccount

            PSDscAllowPlainTextPassword = $true
        }
        @{
            NodeName = "CLDBA.contoso.com"
            SQLFCIs = @(
                @{
                    InstanceName = "MSSQLSERVER"
                    Features = "SQLENGINE,AS,IS"

                    Preferred = 1

                    FailoverClusterNetworkName = "CLSCDB"
                    FailoverClusterIPAddress = "192.168.1.247"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "F:"
                    SQLUserDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "H:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"

                    ASSysAdminAccounts = @("CONTOSO\SQL Admins")
                    ASDataDir = "F:\MSAS11.MSSQLSERVER\OLAP\Data"
                    ASLogDir = "G:\MSAS11.MSSQLSERVER\OLAP\Log"
                    ASBackupDir = "F:\MSAS11.MSSQLSERVER\OLAP\Backup"
                    ASTempDir = "H:\MSAS11.MSSQLSERVER\OLAP\Temp"
                    ASConfigDir = "F:\MSAS11.MSSQLSERVER\OLAP\Config"

                    ISFileSystemFolder = "F:\Packages"
                }
                @{
                    UpdateEnabled = "False"
                    InstanceName = "VMMDB"
                    Features = "SQLENGINE"

                    Preferred = 1

                    FailoverClusterNetworkName = "CLVMMDB"
                    FailoverClusterIPAddress = "192.168.1.246"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "I:"
                    SQLUserDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "K:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SPFDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLSPFDB"
                    FailoverClusterIPAddress = "192.168.1.245"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "L:"
                    SQLUserDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "N:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SMADB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLSMADB"
                    FailoverClusterIPAddress = "192.168.1.244"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "O:"
                    SQLUserDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Q:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "WAPDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLWAPDB"
                    FailoverClusterIPAddress = "192.168.1.243"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "R:"
                    SQLUserDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "T:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLOMDB"
                    FailoverClusterIPAddress = "192.168.1.242"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "U:"
                    SQLUserDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "W:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDW"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLOMDW"
                    FailoverClusterIPAddress = "192.168.1.241"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "X:"
                    SQLUserDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Z:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
            )
        }
        @{
            NodeName = "CLDBB.contoso.com"
            SQLFCIs = @(
                @{
                    InstanceName = "MSSQLSERVER"
                    Features = "SQLENGINE,AS,IS"

                    FailoverClusterNetworkName = "CLSCDB"
                    FailoverClusterIPAddress = "192.168.1.247"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "F:"
                    SQLUserDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "H:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"

                    ASSysAdminAccounts = @("CONTOSO\SQL Admins")
                    ASDataDir = "F:\MSAS11.MSSQLSERVER\OLAP\Data"
                    ASLogDir = "G:\MSAS11.MSSQLSERVER\OLAP\Log"
                    ASBackupDir = "F:\MSAS11.MSSQLSERVER\OLAP\Backup"
                    ASTempDir = "H:\MSAS11.MSSQLSERVER\OLAP\Temp"
                    ASConfigDir = "F:\MSAS11.MSSQLSERVER\OLAP\Config"

                    ISFileSystemFolder = "F:\Packages"
                }
                @{
                    UpdateEnabled = "False"
                    InstanceName = "VMMDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLVMMDB"
                    FailoverClusterIPAddress = "192.168.1.246"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "I:"
                    SQLUserDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "K:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SPFDB"
                    Features = "SQLENGINE"

                    Preferred = 1

                    FailoverClusterNetworkName = "CLSPFDB"
                    FailoverClusterIPAddress = "192.168.1.245"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "L:"
                    SQLUserDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "N:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SMADB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLSMADB"
                    FailoverClusterIPAddress = "192.168.1.244"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "O:"
                    SQLUserDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Q:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "WAPDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLWAPDB"
                    FailoverClusterIPAddress = "192.168.1.243"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "R:"
                    SQLUserDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "T:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLOMDB"
                    FailoverClusterIPAddress = "192.168.1.242"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "U:"
                    SQLUserDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "W:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDW"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLOMDW"
                    FailoverClusterIPAddress = "192.168.1.241"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "X:"
                    SQLUserDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Z:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
            )
        }
        @{
            NodeName = "CLDBC.contoso.com"
            SQLFCIs = @(
                @{
                    InstanceName = "MSSQLSERVER"
                    Features = "SQLENGINE,AS,IS"

                    FailoverClusterNetworkName = "CLSCDB"
                    FailoverClusterIPAddress = "192.168.1.247"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "F:"
                    SQLUserDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "H:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"

                    ASSysAdminAccounts = @("CONTOSO\SQL Admins")
                    ASDataDir = "F:\MSAS11.MSSQLSERVER\OLAP\Data"
                    ASLogDir = "G:\MSAS11.MSSQLSERVER\OLAP\Log"
                    ASBackupDir = "F:\MSAS11.MSSQLSERVER\OLAP\Backup"
                    ASTempDir = "H:\MSAS11.MSSQLSERVER\OLAP\Temp"
                    ASConfigDir = "F:\MSAS11.MSSQLSERVER\OLAP\Config"

                    ISFileSystemFolder = "F:\Packages"
                }
                @{
                    UpdateEnabled = "False"
                    InstanceName = "VMMDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLVMMDB"
                    FailoverClusterIPAddress = "192.168.1.246"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "I:"
                    SQLUserDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "K:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SPFDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLSPFDB"
                    FailoverClusterIPAddress = "192.168.1.245"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "L:"
                    SQLUserDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "N:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SMADB"
                    Features = "SQLENGINE"

                    Preferred = 1

                    FailoverClusterNetworkName = "CLSMADB"
                    FailoverClusterIPAddress = "192.168.1.244"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "O:"
                    SQLUserDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Q:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "WAPDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLWAPDB"
                    FailoverClusterIPAddress = "192.168.1.243"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "R:"
                    SQLUserDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "T:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLOMDB"
                    FailoverClusterIPAddress = "192.168.1.242"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "U:"
                    SQLUserDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "W:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDW"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLOMDW"
                    FailoverClusterIPAddress = "192.168.1.241"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "X:"
                    SQLUserDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Z:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
            )
        }
        @{
            NodeName = "CLDBD.contoso.com"
            SQLFCIs = @(
                @{
                    InstanceName = "MSSQLSERVER"
                    Features = "SQLENGINE,AS,IS"

                    FailoverClusterNetworkName = "CLSCDB"
                    FailoverClusterIPAddress = "192.168.1.247"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "F:"
                    SQLUserDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "H:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"

                    ASSysAdminAccounts = @("CONTOSO\SQL Admins")
                    ASDataDir = "F:\MSAS11.MSSQLSERVER\OLAP\Data"
                    ASLogDir = "G:\MSAS11.MSSQLSERVER\OLAP\Log"
                    ASBackupDir = "F:\MSAS11.MSSQLSERVER\OLAP\Backup"
                    ASTempDir = "H:\MSAS11.MSSQLSERVER\OLAP\Temp"
                    ASConfigDir = "F:\MSAS11.MSSQLSERVER\OLAP\Config"

                    ISFileSystemFolder = "F:\Packages"
                }
                @{
                    UpdateEnabled = "False"
                    InstanceName = "VMMDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLVMMDB"
                    FailoverClusterIPAddress = "192.168.1.246"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "I:"
                    SQLUserDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "K:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SPFDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLSPFDB"
                    FailoverClusterIPAddress = "192.168.1.245"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "L:"
                    SQLUserDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "N:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SMADB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLSMADB"
                    FailoverClusterIPAddress = "192.168.1.244"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "O:"
                    SQLUserDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Q:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "WAPDB"
                    Features = "SQLENGINE"

                    Preferred = 1

                    FailoverClusterNetworkName = "CLWAPDB"
                    FailoverClusterIPAddress = "192.168.1.243"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "R:"
                    SQLUserDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "T:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLOMDB"
                    FailoverClusterIPAddress = "192.168.1.242"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "U:"
                    SQLUserDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "W:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDW"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLOMDW"
                    FailoverClusterIPAddress = "192.168.1.241"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "X:"
                    SQLUserDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Z:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
            )
        }
        @{
            NodeName = "CLDBE.contoso.com"
            SQLFCIs = @(
                @{
                    InstanceName = "MSSQLSERVER"
                    Features = "SQLENGINE,AS,IS"

                    FailoverClusterNetworkName = "CLSCDB"
                    FailoverClusterIPAddress = "192.168.1.247"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "F:"
                    SQLUserDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "H:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"

                    ASSysAdminAccounts = @("CONTOSO\SQL Admins")
                    ASDataDir = "F:\MSAS11.MSSQLSERVER\OLAP\Data"
                    ASLogDir = "G:\MSAS11.MSSQLSERVER\OLAP\Log"
                    ASBackupDir = "F:\MSAS11.MSSQLSERVER\OLAP\Backup"
                    ASTempDir = "H:\MSAS11.MSSQLSERVER\OLAP\Temp"
                    ASConfigDir = "F:\MSAS11.MSSQLSERVER\OLAP\Config"

                    ISFileSystemFolder = "F:\Packages"
                }
                @{
                    UpdateEnabled = "False"
                    InstanceName = "VMMDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLVMMDB"
                    FailoverClusterIPAddress = "192.168.1.246"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "I:"
                    SQLUserDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "K:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SPFDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLSPFDB"
                    FailoverClusterIPAddress = "192.168.1.245"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "L:"
                    SQLUserDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "N:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SMADB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLSMADB"
                    FailoverClusterIPAddress = "192.168.1.244"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "O:"
                    SQLUserDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Q:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "WAPDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLWAPDB"
                    FailoverClusterIPAddress = "192.168.1.243"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "R:"
                    SQLUserDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "T:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDB"
                    Features = "SQLENGINE"

                    Preferred = 1

                    FailoverClusterNetworkName = "CLOMDB"
                    FailoverClusterIPAddress = "192.168.1.242"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "U:"
                    SQLUserDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "W:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDW"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLOMDW"
                    FailoverClusterIPAddress = "192.168.1.241"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "X:"
                    SQLUserDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Z:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
            )
        }
        @{
            NodeName = "CLDBF.contoso.com"
            SQLFCIs = @(
                @{
                    InstanceName = "MSSQLSERVER"
                    Features = "SQLENGINE,AS,IS"

                    FailoverClusterNetworkName = "CLSCDB"
                    FailoverClusterIPAddress = "192.168.1.247"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "F:"
                    SQLUserDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "H:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "G:\MSSQL11.MSSQLSERVER\MSSQL\Data"

                    ASSysAdminAccounts = @("CONTOSO\SQL Admins")
                    ASDataDir = "F:\MSAS11.MSSQLSERVER\OLAP\Data"
                    ASLogDir = "G:\MSAS11.MSSQLSERVER\OLAP\Log"
                    ASBackupDir = "F:\MSAS11.MSSQLSERVER\OLAP\Backup"
                    ASTempDir = "H:\MSAS11.MSSQLSERVER\OLAP\Temp"
                    ASConfigDir = "F:\MSAS11.MSSQLSERVER\OLAP\Config"

                    ISFileSystemFolder = "F:\Packages"
                }
                @{
                    UpdateEnabled = "False"
                    InstanceName = "VMMDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLVMMDB"
                    FailoverClusterIPAddress = "192.168.1.246"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "I:"
                    SQLUserDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "K:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "J:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SPFDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLSPFDB"
                    FailoverClusterIPAddress = "192.168.1.245"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "L:"
                    SQLUserDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "N:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "M:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "SMADB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLSMADB"
                    FailoverClusterIPAddress = "192.168.1.244"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "O:"
                    SQLUserDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Q:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "P:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "WAPDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLWAPDB"
                    FailoverClusterIPAddress = "192.168.1.243"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "R:"
                    SQLUserDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "T:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "S:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDB"
                    Features = "SQLENGINE"

                    FailoverClusterNetworkName = "CLOMDB"
                    FailoverClusterIPAddress = "192.168.1.242"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "U:"
                    SQLUserDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "W:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "V:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
                @{
                    InstanceName = "OMDW"
                    Features = "SQLENGINE"

                    Preferred = 1

                    FailoverClusterNetworkName = "CLOMDW"
                    FailoverClusterIPAddress = "192.168.1.241"

                    SQLSvcAccount = $SQLServiceAccount

                    SQLSysAdminAccounts = @("CONTOSO\SQL Admins")
                    InstallSQLDataDir = "X:"
                    SQLUserDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBDir = "Z:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                    SQLTempDBLogDir = "Y:\MSSQL11.MSSQLSERVER\MSSQL\Data"
                }
            )
        }
    )
}

foreach($Node in $ConfigurationData.AllNodes)
{
    if($Node.NodeName -ne "*")
    {
        Start-Process -FilePath "robocopy.exe" -ArgumentList ("`"C:\Program Files\WindowsPowerShell\Modules`" `"\\" + $Node.NodeName + "\c$\Program Files\WindowsPowerShell\Modules`" /e /purge /xf") -NoNewWindow -Wait
    }
}

SQLFCI -ConfigurationData $ConfigurationData
Set-DscLocalConfigurationManager -Path .\SQLFCI -Verbose
Start-DscConfiguration -Path .\SQLFCI -Wait -Verbose