﻿Configuration WAP2013_HA_Static_AD
{
    Node $AllNodes.NodeName
    {
        LocalConfigurationManager {
            DebugMode = $true
        }
    }

    Import-DscResource -Module xActiveDirectory

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "Active Directory Domain Controller"}}.NodeName
    {
        xADUser "SQL"
        {
            Ensure = "Present"
            DomainName = $Node.SQLServiceAccount.GetNetworkCredential().Domain
            UserName = $Node.SQLServiceAccount.GetNetworkCredential().UserName
            Password = $Node.SQLServiceAccount
            DomainAdministratorCredential = $Node.DomainAdministratorCredential
        }
    }
}

Configuration WAP2013_HA_Static
{
    Node $AllNodes.NodeName
    {
        LocalConfigurationManager {
            DebugMode = $true
        }
    }

    Import-DscResource -Module xPDT

    Node $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("SQL Cluster Node")}}.NodeName
    {
        WindowsFeature "NET-Framework-Core"
        {
            Ensure = "Present"
            Name = "NET-Framework-Core"
            Source = $Node.SourcePath + "\WindowsServer2012R2.en\sources\sxs"
        }

        xSQLFCIPrepare "WAPDB"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
            InstanceName = "WAPDB"
            Features = "SQLENGINE"
            FailoverClusterNetworkName = "CLWAPDB"
            SQLSvcAccount = $Node.SQLServiceAccount
            AgtSvcAccount = $Node.SQLServiceAccount
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "SQL Cluster Node 1"}}.NodeName
    {
        WaitForAll "WAPDB"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("SQL Cluster Node")}}.NodeName
            ResourceName = "[xSQLFCIPrepare]WAPDB"
            RetryIntervalSec = 60
            RetryCount = 120
            Credential = $Node.InstallerServiceAccount
        }

        xSQLFCIComplete "WAPDB"
        {
            DependsOn = "[WaitForAll]WAPDB"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            InstanceName = "WAPDB"
            Features = "SQLENGINE"
            FailoverClusterNetworkName = "CLWAPDB"
            FailoverClusterIPAddress = "192.168.1.243"
            SQLSysAdminAccounts = @("CONTOSO\Administrator")
            InstallSQLDataDir = "O:"
            SQLUserDBLogDir = "P:\MSSQL11.WAPDB\MSSQL\Data"
            SQLTempDBDir = "Q:\MSSQL11.WAPDB\MSSQL\Data"
            SQLTempDBLogDir = "P:\MSSQL11.WAPDB\MSSQL\Data"
            SecurityMode = "SQL"
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("Windows Azure Pack")}}.NodeName
    {
        WindowsFeature "NET-Framework-Core"
        {
            Name = "NET-Framework-Core"
            Ensure = "Present"
            Source = $Node.SourcePath + "\WindowsServer2012R2.en\sources\sxs"
        }

        WindowsFeature "NET-Framework-45-Core"
        {
            Name = "NET-Framework-45-Core"
            Ensure = "Present"
        }

        WindowsFeature "Web-WebServer"
        {
            Ensure = "Present"
            Name = "Web-WebServer"
        }

        WindowsFeature "Web-Default-Doc"
        {
            Ensure = "Present"
            Name = "Web-Default-Doc"
        }

        WindowsFeature "Web-Dir-Browsing"
        {
            Ensure = "Present"
            Name = "Web-Dir-Browsing"
        }

        WindowsFeature "Web-Http-Errors"
        {
            Ensure = "Present"
            Name = "Web-Http-Errors"
        }

        WindowsFeature "Web-Static-Content"
        {
            Ensure = "Present"
            Name = "Web-Static-Content"
        }

        WindowsFeature "Web-Http-Logging"
        {
            Ensure = "Present"
            Name = "Web-Http-Logging"
        }

        WindowsFeature "Web-Stat-Compression"
        {
            Ensure = "Present"
            Name = "Web-Stat-Compression"
        }

        WindowsFeature "Web-Dyn-Compression"
        {
            Ensure = "Present"
            Name = "Web-Dyn-Compression"
        }

        WindowsFeature "Web-Filtering"
        {
            Ensure = "Present"
            Name = "Web-Filtering"
        }

        WindowsFeature "Web-Windows-Auth"
        {
            Ensure = "Present"
            Name = "Web-Windows-Auth"
        }

        WindowsFeature "Web-Asp-Net45"
        {
            Ensure = "Present"
            Name = "Web-Asp-Net45"
        }

        WindowsFeature "Web-Net-Ext45"
        {
            Ensure = "Present"
            Name = "Web-Net-Ext45"
        }

        WindowsFeature "Web-ISAPI-Ext"
        {
            Ensure = "Present"
            Name = "Web-ISAPI-Ext"
        }

        WindowsFeature "Web-ISAPI-Filter"
        {
            Ensure = "Present"
            Name = "Web-ISAPI-Filter"
        }

        WindowsFeature "Web-Mgmt-Compat"
        {
            Ensure = "Present"
            Name = "Web-Mgmt-Compat"
        }

        WindowsFeature "NET-Framework-45-ASPNET"
        {
            Ensure = "Present"
            Name = "NET-Framework-45-ASPNET"
        }

        WindowsFeature "PowerShell"
        {
            Ensure = "Present"
            Name = "PowerShell"
        }

        WindowsFeature "PowerShell-V2"
        {
            Ensure = "Present"
            Name = "PowerShell-V2"
        }

        WindowsFeature "WAS-Process-Model"
        {
            Ensure = "Present"
            Name = "WAS-Process-Model"
        }

        WindowsFeature "WAS-NET-Environment"
        {
            Ensure = "Present"
            Name = "WAS-NET-Environment"
        }

        WindowsFeature "WAS-Config-APIs"
        {
            Ensure = "Present"
            Name = "WAS-Config-APIs"
        }

        WindowsFeature "Web-Mgmt-Console"
        {
            Ensure = "Present"
            Name = "Web-Mgmt-Console"
        }

        xPDTPrerequisiteInstall "VC2010x64"
        {
            Name = "Visual C++ 2010 Redistributable x64"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "VC2010x86"
        {
            Name = "Visual C++ 2010 Redistributable x86"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "ASPNETWebPages2"
        {
            Name = "ASP.NET Web Pages 2"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "ASPNETMVC4"
        {
            Name = "ASP.NET MVC 4"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "MySQLConnectorNet654"
        {
            Name = "MySQL Connector Net 6.5.4"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }

        xPDTPrerequisiteInstall "URLRewrite2"
        {
            Name = "URL Rewrite 2"
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
        }
        
        WaitForAll "WAPDB"
        {
            NodeName = $AllNodes.Where{$_.Roles | Where-Object {$_ -eq "SQL Cluster Node 1"}}.NodeName
            ResourceName = "[xSQLFCIComplete]WAPDB"
            RetryIntervalSec = 60
            RetryCount = 120
            Credential = $Node.InstallerServiceAccount
        }

        xWAP2013Install "WAPAdminAPI"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core",
                "[WindowsFeature]NET-Framework-45-Core",
                "[WindowsFeature]Web-WebServer",
                "[WindowsFeature]Web-Default-Doc",
                "[WindowsFeature]Web-Dir-Browsing",
                "[WindowsFeature]Web-Http-Errors",
                "[WindowsFeature]Web-Static-Content",
                "[WindowsFeature]Web-Http-Logging",
                "[WindowsFeature]Web-Stat-Compression",
                "[WindowsFeature]Web-Dyn-Compression",
                "[WindowsFeature]Web-Filtering",
                "[WindowsFeature]Web-Windows-Auth",
                "[WindowsFeature]Web-Asp-Net45",
                "[WindowsFeature]Web-Net-Ext45",
                "[WindowsFeature]Web-ISAPI-Ext",
                "[WindowsFeature]Web-ISAPI-Filter",
                "[WindowsFeature]Web-Mgmt-Compat",
                "[WindowsFeature]NET-Framework-45-ASPNET",
                "[WindowsFeature]PowerShell",
                "[WindowsFeature]PowerShell-V2",
                "[WindowsFeature]WAS-Process-Model",
                "[WindowsFeature]WAS-NET-Environment",
                "[WindowsFeature]WAS-Config-APIs",
                "[xPDTPrerequisiteInstall]VC2010x64",
                "[xPDTPrerequisiteInstall]VC2010x86",
                "[xPDTPrerequisiteInstall]ASPNETWebPages2",
                "[xPDTPrerequisiteInstall]ASPNETMVC4",
                "[WaitForAll]WAPDB"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            Role = "Admin API Server"
        }

        xWAP2013Install "WAPTenantAPI"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core",
                "[WindowsFeature]NET-Framework-45-Core",
                "[WindowsFeature]Web-WebServer",
                "[WindowsFeature]Web-Default-Doc",
                "[WindowsFeature]Web-Dir-Browsing",
                "[WindowsFeature]Web-Http-Errors",
                "[WindowsFeature]Web-Static-Content",
                "[WindowsFeature]Web-Http-Logging",
                "[WindowsFeature]Web-Stat-Compression",
                "[WindowsFeature]Web-Dyn-Compression",
                "[WindowsFeature]Web-Filtering",
                "[WindowsFeature]Web-Windows-Auth",
                "[WindowsFeature]Web-Asp-Net45",
                "[WindowsFeature]Web-Net-Ext45",
                "[WindowsFeature]Web-ISAPI-Ext",
                "[WindowsFeature]Web-ISAPI-Filter",
                "[WindowsFeature]Web-Mgmt-Compat",
                "[WindowsFeature]NET-Framework-45-ASPNET",
                "[WindowsFeature]PowerShell",
                "[WindowsFeature]PowerShell-V2",
                "[WindowsFeature]WAS-Process-Model",
                "[WindowsFeature]WAS-NET-Environment",
                "[WindowsFeature]WAS-Config-APIs",
                "[xPDTPrerequisiteInstall]VC2010x64",
                "[xPDTPrerequisiteInstall]VC2010x86",
                "[xPDTPrerequisiteInstall]ASPNETWebPages2",
                "[xPDTPrerequisiteInstall]ASPNETMVC4",
                "[WaitForAll]WAPDB"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            Role = "Tenant API Server"
        }

        xWAP2013Install "WAPTenantPublicAPI"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core",
                "[WindowsFeature]NET-Framework-45-Core",
                "[WindowsFeature]Web-WebServer",
                "[WindowsFeature]Web-Default-Doc",
                "[WindowsFeature]Web-Dir-Browsing",
                "[WindowsFeature]Web-Http-Errors",
                "[WindowsFeature]Web-Static-Content",
                "[WindowsFeature]Web-Http-Logging",
                "[WindowsFeature]Web-Stat-Compression",
                "[WindowsFeature]Web-Dyn-Compression",
                "[WindowsFeature]Web-Filtering",
                "[WindowsFeature]Web-Windows-Auth",
                "[WindowsFeature]Web-Asp-Net45",
                "[WindowsFeature]Web-Net-Ext45",
                "[WindowsFeature]Web-ISAPI-Ext",
                "[WindowsFeature]Web-ISAPI-Filter",
                "[WindowsFeature]Web-Mgmt-Compat",
                "[WindowsFeature]NET-Framework-45-ASPNET",
                "[WindowsFeature]PowerShell",
                "[WindowsFeature]PowerShell-V2",
                "[WindowsFeature]WAS-Process-Model",
                "[WindowsFeature]WAS-NET-Environment",
                "[WindowsFeature]WAS-Config-APIs",
                "[xPDTPrerequisiteInstall]VC2010x64",
                "[xPDTPrerequisiteInstall]VC2010x86",
                "[xPDTPrerequisiteInstall]ASPNETWebPages2",
                "[xPDTPrerequisiteInstall]ASPNETMVC4",
                "[WaitForAll]WAPDB"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            Role = "Tenant Public API Server"
        }

        xWAP2013Install "WAPSQLServerExtension"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core",
                "[WindowsFeature]NET-Framework-45-Core",
                "[WindowsFeature]Web-WebServer",
                "[WindowsFeature]Web-Default-Doc",
                "[WindowsFeature]Web-Dir-Browsing",
                "[WindowsFeature]Web-Http-Errors",
                "[WindowsFeature]Web-Static-Content",
                "[WindowsFeature]Web-Http-Logging",
                "[WindowsFeature]Web-Stat-Compression",
                "[WindowsFeature]Web-Dyn-Compression",
                "[WindowsFeature]Web-Filtering",
                "[WindowsFeature]Web-Windows-Auth",
                "[WindowsFeature]Web-Asp-Net45",
                "[WindowsFeature]Web-Net-Ext45",
                "[WindowsFeature]Web-ISAPI-Ext",
                "[WindowsFeature]Web-ISAPI-Filter",
                "[WindowsFeature]Web-Mgmt-Compat",
                "[WindowsFeature]NET-Framework-45-ASPNET",
                "[WindowsFeature]PowerShell",
                "[WindowsFeature]PowerShell-V2",
                "[WindowsFeature]WAS-Process-Model",
                "[WindowsFeature]WAS-NET-Environment",
                "[WindowsFeature]WAS-Config-APIs",
                "[xPDTPrerequisiteInstall]VC2010x64",
                "[xPDTPrerequisiteInstall]VC2010x86",
                "[xPDTPrerequisiteInstall]ASPNETWebPages2",
                "[xPDTPrerequisiteInstall]ASPNETMVC4",
                "[WaitForAll]WAPDB"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            Role = "SQL Server Extension Server"
        }

        xWAP2013Install "WAPMySQLExtension"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core",
                "[WindowsFeature]NET-Framework-45-Core",
                "[WindowsFeature]Web-WebServer",
                "[WindowsFeature]Web-Default-Doc",
                "[WindowsFeature]Web-Dir-Browsing",
                "[WindowsFeature]Web-Http-Errors",
                "[WindowsFeature]Web-Static-Content",
                "[WindowsFeature]Web-Http-Logging",
                "[WindowsFeature]Web-Stat-Compression",
                "[WindowsFeature]Web-Dyn-Compression",
                "[WindowsFeature]Web-Filtering",
                "[WindowsFeature]Web-Windows-Auth",
                "[WindowsFeature]Web-Asp-Net45",
                "[WindowsFeature]Web-Net-Ext45",
                "[WindowsFeature]Web-ISAPI-Ext",
                "[WindowsFeature]Web-ISAPI-Filter",
                "[WindowsFeature]Web-Mgmt-Compat",
                "[WindowsFeature]NET-Framework-45-ASPNET",
                "[WindowsFeature]PowerShell",
                "[WindowsFeature]PowerShell-V2",
                "[WindowsFeature]WAS-Process-Model",
                "[WindowsFeature]WAS-NET-Environment",
                "[WindowsFeature]WAS-Config-APIs",
                "[xPDTPrerequisiteInstall]VC2010x64",
                "[xPDTPrerequisiteInstall]VC2010x86",
                "[xPDTPrerequisiteInstall]ASPNETWebPages2",
                "[xPDTPrerequisiteInstall]ASPNETMVC4",
                "[xPDTPrerequisiteInstall]MySQLConnectorNet654"
                "[WaitForAll]WAPDB"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            Role = "MySQL Extension Server"
        }

        xWAP2013Install "WAPAdminSite"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core",
                "[WindowsFeature]NET-Framework-45-Core",
                "[WindowsFeature]Web-WebServer",
                "[WindowsFeature]Web-Default-Doc",
                "[WindowsFeature]Web-Dir-Browsing",
                "[WindowsFeature]Web-Http-Errors",
                "[WindowsFeature]Web-Static-Content",
                "[WindowsFeature]Web-Http-Logging",
                "[WindowsFeature]Web-Stat-Compression",
                "[WindowsFeature]Web-Dyn-Compression",
                "[WindowsFeature]Web-Filtering",
                "[WindowsFeature]Web-Windows-Auth",
                "[WindowsFeature]Web-Asp-Net45",
                "[WindowsFeature]Web-Net-Ext45",
                "[WindowsFeature]Web-ISAPI-Ext",
                "[WindowsFeature]Web-ISAPI-Filter",
                "[WindowsFeature]Web-Mgmt-Compat",
                "[WindowsFeature]NET-Framework-45-ASPNET",
                "[WindowsFeature]PowerShell",
                "[WindowsFeature]PowerShell-V2",
                "[WindowsFeature]WAS-Process-Model",
                "[WindowsFeature]WAS-NET-Environment",
                "[WindowsFeature]WAS-Config-APIs",
                "[WindowsFeature]Web-Mgmt-Console",
                "[xPDTPrerequisiteInstall]VC2010x64",
                "[xPDTPrerequisiteInstall]VC2010x86",
                "[xPDTPrerequisiteInstall]ASPNETWebPages2",
                "[xPDTPrerequisiteInstall]ASPNETMVC4",
                "[xPDTPrerequisiteInstall]URLRewrite2",
                "[WaitForAll]WAPDB"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            Role = "Admin Site Server"
        }

        xWAP2013Install "WAPAdminAuthSite"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core",
                "[WindowsFeature]NET-Framework-45-Core",
                "[WindowsFeature]Web-WebServer",
                "[WindowsFeature]Web-Default-Doc",
                "[WindowsFeature]Web-Dir-Browsing",
                "[WindowsFeature]Web-Http-Errors",
                "[WindowsFeature]Web-Static-Content",
                "[WindowsFeature]Web-Http-Logging",
                "[WindowsFeature]Web-Stat-Compression",
                "[WindowsFeature]Web-Dyn-Compression",
                "[WindowsFeature]Web-Filtering",
                "[WindowsFeature]Web-Windows-Auth",
                "[WindowsFeature]Web-Asp-Net45",
                "[WindowsFeature]Web-Net-Ext45",
                "[WindowsFeature]Web-ISAPI-Ext",
                "[WindowsFeature]Web-ISAPI-Filter",
                "[WindowsFeature]Web-Mgmt-Compat",
                "[WindowsFeature]NET-Framework-45-ASPNET",
                "[WindowsFeature]PowerShell",
                "[WindowsFeature]PowerShell-V2",
                "[WindowsFeature]WAS-Process-Model",
                "[WindowsFeature]WAS-NET-Environment",
                "[WindowsFeature]WAS-Config-APIs",
                "[WindowsFeature]Web-Mgmt-Console",
                "[xPDTPrerequisiteInstall]VC2010x64",
                "[xPDTPrerequisiteInstall]VC2010x86",
                "[xPDTPrerequisiteInstall]ASPNETWebPages2",
                "[xPDTPrerequisiteInstall]ASPNETMVC4",
                "[xPDTPrerequisiteInstall]URLRewrite2",
                "[WaitForAll]WAPDB"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            Role = "Admin Authentication Site Server"
        }

        xWAP2013Install "WAPTenantSite"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core",
                "[WindowsFeature]NET-Framework-45-Core",
                "[WindowsFeature]Web-WebServer",
                "[WindowsFeature]Web-Default-Doc",
                "[WindowsFeature]Web-Dir-Browsing",
                "[WindowsFeature]Web-Http-Errors",
                "[WindowsFeature]Web-Static-Content",
                "[WindowsFeature]Web-Http-Logging",
                "[WindowsFeature]Web-Stat-Compression",
                "[WindowsFeature]Web-Dyn-Compression",
                "[WindowsFeature]Web-Filtering",
                "[WindowsFeature]Web-Windows-Auth",
                "[WindowsFeature]Web-Asp-Net45",
                "[WindowsFeature]Web-Net-Ext45",
                "[WindowsFeature]Web-ISAPI-Ext",
                "[WindowsFeature]Web-ISAPI-Filter",
                "[WindowsFeature]Web-Mgmt-Compat",
                "[WindowsFeature]NET-Framework-45-ASPNET",
                "[WindowsFeature]PowerShell",
                "[WindowsFeature]PowerShell-V2",
                "[WindowsFeature]WAS-Process-Model",
                "[WindowsFeature]WAS-NET-Environment",
                "[WindowsFeature]WAS-Config-APIs",
                "[WindowsFeature]Web-Mgmt-Console",
                "[xPDTPrerequisiteInstall]VC2010x64",
                "[xPDTPrerequisiteInstall]VC2010x86",
                "[xPDTPrerequisiteInstall]ASPNETWebPages2",
                "[xPDTPrerequisiteInstall]ASPNETMVC4",
                "[xPDTPrerequisiteInstall]URLRewrite2",
                "[WaitForAll]WAPDB"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            Role = "Tenant Site Server"
        }

        xWAP2013Install "WAPTenantAuthSite"
        {
            DependsOn = @(
                "[WindowsFeature]NET-Framework-Core",
                "[WindowsFeature]NET-Framework-45-Core",
                "[WindowsFeature]Web-WebServer",
                "[WindowsFeature]Web-Default-Doc",
                "[WindowsFeature]Web-Dir-Browsing",
                "[WindowsFeature]Web-Http-Errors",
                "[WindowsFeature]Web-Static-Content",
                "[WindowsFeature]Web-Http-Logging",
                "[WindowsFeature]Web-Stat-Compression",
                "[WindowsFeature]Web-Dyn-Compression",
                "[WindowsFeature]Web-Filtering",
                "[WindowsFeature]Web-Windows-Auth",
                "[WindowsFeature]Web-Asp-Net45",
                "[WindowsFeature]Web-Net-Ext45",
                "[WindowsFeature]Web-ISAPI-Ext",
                "[WindowsFeature]Web-ISAPI-Filter",
                "[WindowsFeature]Web-Mgmt-Compat",
                "[WindowsFeature]NET-Framework-45-ASPNET",
                "[WindowsFeature]PowerShell",
                "[WindowsFeature]PowerShell-V2",
                "[WindowsFeature]WAS-Process-Model",
                "[WindowsFeature]WAS-NET-Environment",
                "[WindowsFeature]WAS-Config-APIs",
                "[WindowsFeature]Web-Mgmt-Console",
                "[xPDTPrerequisiteInstall]VC2010x64",
                "[xPDTPrerequisiteInstall]VC2010x86",
                "[xPDTPrerequisiteInstall]ASPNETWebPages2",
                "[xPDTPrerequisiteInstall]ASPNETMVC4",
                "[xPDTPrerequisiteInstall]URLRewrite2",
                "[WaitForAll]WAPDB"
            )
            SourcePath = $Node.SourcePath
            Credential = $Node.InstallerServiceAccount
            Role = "Tenant Authentication Site Server"
        }

        xWAP2013Config "WAPAdminAPI"
        {
            DependsOn = "[xWAP2013Install]WAPAdminAPI"
            Credential = $Node.InstallerServiceAccount
            Role = "Admin API Server"
            DatabaseServer = "CLWAPDB\WAPDB"
        }

        xWAP2013Config "WAPTenantAPI"
        {
            DependsOn = "[xWAP2013Install]WAPTenantAPI"
            Credential = $Node.InstallerServiceAccount
            Role = "Tenant API Server"
            DatabaseServer = "CLWAPDB\WAPDB"
        }

        xWAP2013Config "WAPTenantPublicAPI"
        {
            DependsOn = "[xWAP2013Install]WAPTenantPublicAPI"
            Credential = $Node.InstallerServiceAccount
            Role = "Tenant Public API Server"
            DatabaseServer = "CLWAPDB\WAPDB"
        }

        xWAP2013Config "WAPSQLServerExtension"
        {
            DependsOn = "[xWAP2013Install]WAPSQLServerExtension"
            Credential = $Node.InstallerServiceAccount
            Role = "SQL Server Extension Server"
            DatabaseServer = "CLWAPDB\WAPDB"
        }

        xWAP2013Config "WAPMySQLExtension"
        {
            DependsOn = "[xWAP2013Install]WAPMySQLExtension"
            Credential = $Node.InstallerServiceAccount
            Role = "MySQL Extension Server"
            DatabaseServer = "CLWAPDB\WAPDB"
        }

        xWAP2013Config "WAPAdminSite"
        {
            DependsOn = "[xWAP2013Install]WAPAdminSite"
            Credential = $Node.InstallerServiceAccount
            Role = "Admin Site Server"
            DatabaseServer = "CLWAPDB\WAPDB"
        }

        xWAP2013Config "WAPAdminAuthSite"
        {
            DependsOn = "[xWAP2013Install]WAPAdminAuthSite"
            Credential = $Node.InstallerServiceAccount
            Role = "Admin Authentication Site Server"
            DatabaseServer = "CLWAPDB\WAPDB"
        }

        xWAP2013Config "WAPTenantSite"
        {
            DependsOn = "[xWAP2013Install]WAPTenantSite"
            Credential = $Node.InstallerServiceAccount
            Role = "Tenant Site Server"
            DatabaseServer = "CLWAPDB\WAPDB"
        }

        xWAP2013Config "WAPTenantAuthSite"
        {
            DependsOn = "[xWAP2013Install]WAPTenantAuthSite"
            Credential = $Node.InstallerServiceAccount
            Role = "Tenant Authentication Site Server"
            DatabaseServer = "CLWAPDB\WAPDB"
        }
    }

    Node $AllNodes.Where{$_.Roles | Where-Object {$_.Contains("Console Node")}}.NodeName
    {
        WindowsFeature "NET-Framework-Core"
        {
            Name = "NET-Framework-Core"
            Ensure = "Present"
            Source = $Node.SourcePath +  "\WindowsServer2012R2\sources\sxs"
        }

        xSQLServerInstall "SQLMT"
        {
            DependsOn = "[WindowsFeature]NET-Framework-Core"
            SourcePath = $Node.SourcePath
            InstanceName = "NULL"
            Features = "SSMS,ADV_SSMS"
            Credential = $Node.InstallerServiceAccount
            PID = $Node.SQLServer2012ProductKey
        }
    }
}

$SecurePassword = ConvertTo-SecureString -String "Pass@word1" -AsPlainText -Force
$DomainAdministratorCredential = New-Object System.Management.Automation.PSCredential ("CONTOSO\Administrator", $SecurePassword)
$InstallerServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!Installer", $SecurePassword)
$SQLServiceAccount = New-Object System.Management.Automation.PSCredential ("CONTOSO\!sql", $SecurePassword)

$ConfigurationData = @{
    AllNodes = @(
        @{
            NodeName = "*"
            PSDscAllowPlainTextPassword = $true

            DomainAdministratorCredential = $DomainAdministratorCredential
            InstallerServiceAccount = $InstallerServiceAccount

            SQLServer2012ProductKey = "748RB-X4T6B-MRM7V-RTVFF-CHC8H"
            SystemCenter2012ProductKey = "BXH69-M62YX-QQD6R-3GPWX-8WMFY"

            SourcePath = "\\RD01\Installer"

            RegisteredUser = "Microsoft Corporation"
            RegisteredOrganization = "Microsoft Corporation"
            
            SQLServiceAccount = $SQLServiceAccount
        }
        @{
            NodeName = "DC01.contoso.com"
            Roles = @("Active Directory Domain Controller")
        }
        @{
            NodeName = "CLDBA.contoso.com"
            Roles = @("SQL Cluster Node 1")
        }
        @{
            NodeName = "CLDBB.contoso.com"
            Roles = @("SQL Cluster Node 2")
        }
        @{
            NodeName = "CLDBC.contoso.com"
            Roles = @("SQL Cluster Node 3")
        }
        @{
            NodeName = "CLDBD.contoso.com"
            Roles = @("SQL Cluster Node 4")
        }
        @{
            NodeName = "WAPA01.contoso.com"
            Roles = @("Windows Azure Pack Admin API Server")
        }
        @{
            NodeName = "RD01.contoso.com"
            Roles = @("Console Node 1")
        }
    )
}

foreach($Node in $ConfigurationData.AllNodes)
{
    if($Node.NodeName -ne "*")
    {
        Start-Process -FilePath "robocopy.exe" -ArgumentList ("`"C:\Program Files\WindowsPowerShell\Modules`" `"\\" + $Node.NodeName + "\c$\Program Files\WindowsPowerShell\Modules`" /e /purge /xf") -NoNewWindow -Wait
    }
}

WAP2013_HA_Static_AD -ConfigurationData $ConfigurationData
WAP2013_HA_Static -ConfigurationData $ConfigurationData
Set-DscLocalConfigurationManager -Path .\WAP2013_HA_Static_AD -Verbose
Set-DscLocalConfigurationManager -Path .\WAP2013_HA_Static -Verbose
Start-DscConfiguration -Path .\WAP2013_HA_Static_AD -Verbose -Wait
Start-DscConfiguration -Path .\WAP2013_HA_Static -Verbose -Wait